
#import "CIPalEffectFilter.h"


@implementation CIPalEffectFilter


static CIKernel* sPalEffectKernal = nil;

static NSString* sPalEffectKernalCode = @"															\
kernel vec4 palEffect(sampler source_image)															\
{																									\
	vec2 xy = destCoord();																	\
	vec4 pixel   = sample(source_image, samplerTransform(source_image, xy+vec2( 0.0,  0.0)));		\
	vec4 pixel_l = sample(source_image, samplerTransform(source_image, xy+vec2(-1.0,  0.0)));		\
	vec4 pixel_r = sample(source_image, samplerTransform(source_image, xy+vec2(+1.0,  0.0)));		\
	vec4 pixel_u = sample(source_image, samplerTransform(source_image, xy+vec2( 0.0, -1.0)));		\
	vec4 yuv_pixel, yuv_pixel_l, yuv_pixel_r, yuv_pixel_u;											\
																									\
	yuv_pixel.r = clamp(( 0.299*pixel.r + 0.587*pixel.g + 0.114*pixel.b      ), 0.0, 1.0);			\
	yuv_pixel.g = clamp((-0.147*pixel.r - 0.289*pixel.g + 0.436*pixel.b + 0.5), 0.0, 1.0);			\
	yuv_pixel.b = clamp(( 0.615*pixel.r - 0.515*pixel.g - 0.100*pixel.b + 0.5), 0.0, 1.0);			\
																									\
	yuv_pixel_l.r = clamp(( 0.299*pixel_l.r + 0.587*pixel_l.g + 0.114*pixel_l.b), 0.0, 1.0);		\
	yuv_pixel_l.g = clamp((-0.147*pixel_l.r - 0.289*pixel_l.g + 0.436*pixel_l.b + 0.5), 0.0, 1.0);	\
	yuv_pixel_l.b = clamp(( 0.615*pixel_l.r - 0.515*pixel_l.g - 0.100*pixel_l.b + 0.5), 0.0, 1.0);	\
																									\
	yuv_pixel_r.r = clamp(( 0.299*pixel_r.r + 0.587*pixel_r.g + 0.114*pixel_r.b), 0.0, 1.0);		\
	yuv_pixel_r.g = clamp((-0.147*pixel_r.r - 0.289*pixel_r.g + 0.436*pixel_r.b + 0.5), 0.0, 1.0);	\
	yuv_pixel_r.b = clamp(( 0.615*pixel_r.r - 0.515*pixel_r.g - 0.100*pixel_r.b + 0.5), 0.0, 1.0);	\
																									\
	yuv_pixel_u.r = clamp(( 0.299*pixel_u.r + 0.587*pixel_u.g + 0.114*pixel_u.b), 0.0, 1.0);		\
	yuv_pixel_u.g = clamp((-0.147*pixel_u.r - 0.289*pixel_u.g + 0.436*pixel_u.b + 0.5), 0.0, 1.0);	\
	yuv_pixel_u.b = clamp(( 0.615*pixel_u.r - 0.515*pixel_u.g - 0.100*pixel_u.b + 0.5), 0.0, 1.0);	\
																									\
	yuv_pixel.r = (yuv_pixel_l.r + yuv_pixel.r*5.0 + yuv_pixel_r.r) / 7.0;							\
	yuv_pixel.g = (yuv_pixel_l.g + yuv_pixel.g     + yuv_pixel_r.g) / 3.0;							\
	yuv_pixel.b = (yuv_pixel_l.b + yuv_pixel.b     + yuv_pixel_r.b) / 3.0;							\
																									\
	yuv_pixel.r -= mod(xy.y, 2.0) * 0.01;															\
	yuv_pixel.g = (yuv_pixel.g + yuv_pixel_u.g) / 2.0 - 0.5;										\
	yuv_pixel.b = (yuv_pixel.b + yuv_pixel_u.b) / 2.0 - 0.5;										\
																									\
	pixel.r = clamp(yuv_pixel.r                     + 1.140*yuv_pixel.b, 0.0, 1.0);					\
	pixel.g = clamp(yuv_pixel.r - 0.395*yuv_pixel.g - 0.581*yuv_pixel.b, 0.0, 1.0);					\
	pixel.b = clamp(yuv_pixel.r + 2.032*yuv_pixel.g                    , 0.0, 1.0);					\
																									\
	return pixel;																					\
}";


// ----------------------------------------------------------------------------
+ (void) initialize
// ----------------------------------------------------------------------------
{
	NSArray* categories = [NSArray arrayWithObjects:kCICategoryColorAdjustment, kCICategoryVideo, kCICategoryStillImage, kCICategoryInterlaced, kCICategoryNonSquarePixels, nil];
	[CIFilter registerFilterName:@"CIPalEffect" constructor: self classAttributes: [NSDictionary dictionaryWithObjectsAndKeys:@"PAL Effect", kCIAttributeFilterDisplayName, 
																															   categories, kCIAttributeFilterCategories, nil]];
}


// ----------------------------------------------------------------------------
+ (CIFilter*) filterWithName:(NSString*)name
// ----------------------------------------------------------------------------
{
    return [[self alloc] init];
}


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
    if (sPalEffectKernal == nil)
    {
        NSArray* kernels = [CIKernel kernelsWithString:sPalEffectKernalCode];
        sPalEffectKernal = [kernels objectAtIndex:0];
    }
	
    return [super init];
}


// ----------------------------------------------------------------------------
- (CIImage*) outputImage
// ----------------------------------------------------------------------------
{
    CISampler* inputSampler = [CISampler samplerWithImage:inputImage];
	CIImage* outputImage = [self apply:sPalEffectKernal, inputSampler, kCIApplyOptionDefinition, [inputSampler definition], nil];
	
    return outputImage;
}

@end
