
#import "GBExtrasBrowserDataSource.h"
#import "NSApplicationAdditions.h"


@implementation GBExtrasBrowserItem

@synthesize path = _path;
@synthesize title = _title;
@synthesize subTitle = _subTitle;


static NSUInteger sImageVersion = 0;


// ----------------------------------------------------------------------------
- (id) initWithPath:(NSString*)thePath andTitle:(NSString*)theTitle andSubTitle:(NSString*)theSubTitle
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		_path = thePath;
		_title = theTitle;
		_subTitle = theSubTitle;
		sImageVersion++;
	}
	return self;
}


// IKImageBrowserItem protocol

// ----------------------------------------------------------------------------
- (NSString*) imageRepresentationType
// ----------------------------------------------------------------------------
{
	return IKImageBrowserPathRepresentationType;
}


// ----------------------------------------------------------------------------
- (id) imageRepresentation
// ----------------------------------------------------------------------------
{
	return _path;
}


// ----------------------------------------------------------------------------
- (NSString*) imageUID
// ----------------------------------------------------------------------------
{
	return _path;
}


// ----------------------------------------------------------------------------
- (NSString *) imageTitle
// ----------------------------------------------------------------------------
{
	return _title;
}


// ----------------------------------------------------------------------------
- (NSString *) imageSubtitle
// ----------------------------------------------------------------------------
{
	return _subTitle;
}


// ----------------------------------------------------------------------------
- (NSUInteger)imageVersion
// ----------------------------------------------------------------------------
{
	return sImageVersion;
}


// ----------------------------------------------------------------------------
- (NSURL*) previewItemURL
// ----------------------------------------------------------------------------
{
	return [NSURL fileURLWithPath:_path];
}


// ----------------------------------------------------------------------------
- (NSString*) previewItemTitle
// ----------------------------------------------------------------------------
{
	return _title;
}

@end

#pragma mark -

@implementation GBExtrasBrowserDataSource


@synthesize browserData = _browserData;
@synthesize browserView = _browserView;


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	_browserData = [NSMutableArray arrayWithCapacity:10];
}


// ----------------------------------------------------------------------------
- (NSUInteger) numberOfItemsInImageBrowser:(IKImageBrowserView*)view
// ----------------------------------------------------------------------------
{
    return [_browserData count];
}


// ----------------------------------------------------------------------------
- (id) imageBrowser:(IKImageBrowserView*)view itemAtIndex:(NSUInteger)index
// ----------------------------------------------------------------------------
{
    return [_browserData objectAtIndex:index];
}


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)view removeItemsAtIndexes:(NSIndexSet*)indexes
// ----------------------------------------------------------------------------
{
	[_browserData removeObjectsAtIndexes:indexes];
}



// delegate protocol


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)browser cellWasRightClickedAtIndex:(NSUInteger)index withEvent:(NSEvent*)event;
// ----------------------------------------------------------------------------
{
	NSMenu* menu = [browser menuForEvent:event];
	[NSMenu popUpContextMenu:menu withEvent:event forView:browser];
	
}


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)browser cellWasDoubleClickedAtIndex:(NSUInteger)index
// ----------------------------------------------------------------------------
{
	GBExtrasBrowserItem* item = [_browserData objectAtIndex:index];
	NSWorkspace *workSpace = [NSWorkspace sharedWorkspace];

	if ([browser window].currentEvent.modifierFlags & NSCommandKeyMask)
		[workSpace selectFile:item.path inFileViewerRootedAtPath:@""];
	else
		[workSpace openFile:item.path];
}


// ----------------------------------------------------------------------------
- (void) imageBrowserSelectionDidChange:(IKImageBrowserView*)browser
// ----------------------------------------------------------------------------
{
	NSUInteger selectedIndex = [[browser selectionIndexes] firstIndex];
	if (selectedIndex != NSNotFound)
	{
		if (_browserView.previewPanel != nil)
			[_browserView.previewPanel reloadData];
	}
}


// ----------------------------------------------------------------------------
- (IBAction) revealExtraItemInFinder:(id)sender
// ----------------------------------------------------------------------------
{
	NSUInteger selectedIndex = [[_browserView selectionIndexes] firstIndex];
	if (selectedIndex == NSNotFound)
		return;

	GBExtrasBrowserItem* item = [_browserData objectAtIndex:selectedIndex];
	if (item != nil)
	{
		NSWorkspace *workSpace = [NSWorkspace sharedWorkspace];
		[workSpace selectFile:item.path inFileViewerRootedAtPath:@""];
	}
}


// ----------------------------------------------------------------------------
- (IBAction) copyPathOfExtraItemToClipboard:(id)sender
// ----------------------------------------------------------------------------
{
	NSUInteger selectedIndex = [[_browserView selectionIndexes] firstIndex];
	if (selectedIndex == NSNotFound)
		return;
	
	GBExtrasBrowserItem* item = [_browserData objectAtIndex:selectedIndex];
	if (item != nil)
	{
		NSPasteboard* pasteboard = [NSPasteboard generalPasteboard];
		[pasteboard declareTypes:[NSArray arrayWithObjects:NSStringPboardType, nil] owner:self];
		[pasteboard setString:item.path forType:NSStringPboardType];
	}
}


#pragma mark -
#pragma mark Quick Look panel data source


// ----------------------------------------------------------------------------
- (NSInteger) numberOfPreviewItemsInPreviewPanel:(QLPreviewPanel*)panel
// ----------------------------------------------------------------------------
{
    return 1;
}


// ----------------------------------------------------------------------------
- (id <QLPreviewItem>) previewPanel:(QLPreviewPanel*)panel previewItemAtIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	NSUInteger selectedIndex = [[_browserView selectionIndexes] firstIndex];
	if (selectedIndex != NSNotFound)
		return [_browserData objectAtIndex:selectedIndex];
	else
		return nil;
}


#pragma mark -
#pragma mark Quick Look panel delegate


// ----------------------------------------------------------------------------
- (BOOL) previewPanel:(QLPreviewPanel*)panel handleEvent:(NSEvent*)event
// ----------------------------------------------------------------------------
{
    // redirect all key down events to the browser view
    if ([event type] == NSKeyDown)
	{
        [_browserView keyDown:event];
        return YES;
    }
    return NO;
}


// ----------------------------------------------------------------------------
- (NSRect) previewPanel:(QLPreviewPanel*)panel sourceFrameOnScreenForPreviewItem:(id <QLPreviewItem>)item
// ----------------------------------------------------------------------------
{
    NSInteger index = [_browserData indexOfObject:item];
    if (index == NSNotFound)
        return NSZeroRect;
	
    NSRect iconRect = [_browserView itemFrameAtIndex:index];
    
    // check that the icon rect is visible on screen
    NSRect visibleRect = [_browserView visibleRect];
    
    if (!NSIntersectsRect(visibleRect, iconRect))
        return NSZeroRect;
    
    // convert icon rect to screen coordinates
	iconRect = [_browserView convertRect:iconRect toView:[[_browserView window] contentView]];
    iconRect = [[_browserView window] convertRectToScreen:iconRect];
    
    return iconRect;
}


// ----------------------------------------------------------------------------
- (id) previewPanel:(QLPreviewPanel*)panel transitionImageForPreviewItem:(id <QLPreviewItem>)item contentRect:(NSRect*)contentRect
// ----------------------------------------------------------------------------
{
	return nil;
}


@end

#pragma mark -

@implementation GBExtrasBrowserView

@synthesize	previewPanel = _previewPanel;

// ----------------------------------------------------------------------------
- (NSMenu*) menuForEvent:(NSEvent*)event
// ----------------------------------------------------------------------------
{
	NSPoint position = [self convertPoint:[event locationInWindow] fromView:nil];
	NSInteger row = [self indexOfItemAtPoint:position];
	
	if (row != -1)
	{
		NSIndexSet* set = [self selectionIndexes];
		if (![set containsIndex:row])
			[self setSelectionIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];

		return contextMenu;
	}
		
	return [super menuForEvent:event];
}


// ----------------------------------------------------------------------------
- (void) keyDown:(NSEvent *)event
// ----------------------------------------------------------------------------
{	
	if (![NSApp isOnSnowLeopardOrBetter])
	{
		[super keyDown:event];
		return;
	}
	
	if([[event charactersIgnoringModifiers] characterAtIndex:0] == ' ')
	{
		if ([[QLPreviewPanelSL sharedPreviewPanel] isVisible])
			[[QLPreviewPanelSL sharedPreviewPanel] orderOut:nil];
		else
			[[QLPreviewPanelSL sharedPreviewPanel] makeKeyAndOrderFront:nil];
	}
	else
		[super keyDown:event];
}


#pragma mark -
#pragma mark Quick Look panel support


// ----------------------------------------------------------------------------
- (BOOL) acceptsPreviewPanelControl:(QLPreviewPanel*)panel
// ----------------------------------------------------------------------------
{
    return YES;
}


// ----------------------------------------------------------------------------
- (void) beginPreviewPanelControl:(QLPreviewPanel*)panel
// ----------------------------------------------------------------------------
{
    _previewPanel = panel;
    panel.delegate = [self dataSource];
    panel.dataSource = [self dataSource];
}


// ----------------------------------------------------------------------------
- (void) endPreviewPanelControl:(QLPreviewPanel*)panel
// ----------------------------------------------------------------------------
{
    _previewPanel = nil;
}
 

@end


