
#import "GBGameCollection.h"
#import "GameBaseAppDelegate.h"


@implementation GBGameCollection

@synthesize name = _name;
@synthesize identifier = _identifier;
@synthesize path = _path;
@synthesize gameIds = _gameIds;
@synthesize isSmart = _isSmart;


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		_gameIds = [NSMutableArray arrayWithCapacity:16];
		_name = nil;
        _isSmart = NO;
		
		CFUUIDRef uuid = CFUUIDCreate(NULL);
		_identifier = (__bridge_transfer NSString *) CFUUIDCreateString(NULL, uuid);
		CFRelease(uuid);
	}
	return self;
}


// ----------------------------------------------------------------------------
- (id) initWithCoder:(NSCoder*)coder
// ----------------------------------------------------------------------------
{
	if (self = [super init])
	{
		if ([coder allowsKeyedCoding])
		{
			self.name = [coder decodeObjectForKey:@"GBKeyName"];
			self.identifier = [coder decodeObjectForKey:@"GBKeyIdentifier"];
			self.gameIds = [coder decodeObjectForKey:@"GBKeyGameIds"];
		}
		else
		{
			self.name = [coder decodeObject];
			self.identifier = [coder decodeObject];
			self.gameIds = [coder decodeObject];
		}
	}
	return self;
}


// ----------------------------------------------------------------------------
- (void) encodeWithCoder:(NSCoder*)coder
// ----------------------------------------------------------------------------
{
    if ([coder allowsKeyedCoding])
	{
		[coder encodeObject:_name forKey:@"GBKeyName"];
		[coder encodeObject:_identifier forKey:@"GBKeyIdentifier"];
        [coder encodeObject:_gameIds forKey:@"GBKeyGameIds"];
	}
	else
	{
		[coder encodeObject:_name];
		[coder encodeObject:_identifier];
		[coder encodeObject:_gameIds];
	}
}


// ----------------------------------------------------------------------------
- (NSData*) dataRepresentation
// ----------------------------------------------------------------------------
{
	return [NSKeyedArchiver archivedDataWithRootObject:self];
}


// ----------------------------------------------------------------------------
- (BOOL) saveToFile
// ----------------------------------------------------------------------------
{
    if (_isSmart)
        return NO;
    
	NSString* filename = [_identifier stringByAppendingPathExtension:[GBGameCollection fileExtension]];
	_path = [[GameBaseAppDelegate applicationSupportPath] stringByAppendingPathComponent:filename];
	
	NSData* data = [self dataRepresentation];
	BOOL success = [data writeToFile:_path atomically:YES];
	return success;
}


// ----------------------------------------------------------------------------
+ (NSString*) fileExtension
// ----------------------------------------------------------------------------
{
	static NSString* extension = @"gbcollection";
	
	return extension;
}


// ----------------------------------------------------------------------------
+ (GBGameCollection*) collectionFromData:(NSData*)data
// ----------------------------------------------------------------------------
{
	return [NSKeyedUnarchiver unarchiveObjectWithData:data];
}


// ----------------------------------------------------------------------------
+ (GBGameCollection*) collectionFromFile:(NSString*)path
// ----------------------------------------------------------------------------
{
	NSData* data = [NSData dataWithContentsOfFile:path];
	GBGameCollection* collection = [GBGameCollection collectionFromData:data];
	collection.path = path;
	
	return collection;
}


@end

