
#import "GBGameDetailController.h"
#import "GameBaseAppDelegate.h"
#import "GBSourceListItem.h"
#import "GBSourceListDataSource.h"
#import "GBPreferencesController.h"
#import "GBExtrasBrowserDataSource.h"
#import "GBGameTableDataSource.h"
#import "GBDatabaseFileDownloadController.h"
#import "CIPalEffectFilter.h"
#import "CICCS64ColorCorrectionFilter.h"

#import <objc/objc-class.h>


//#define ROTATE_TRANSITION


@interface CIContext(CS)

@end

@implementation CIContext(CS)

// ----------------------------------------------------------------------------
+ (CIContext*) real_contextWithCGLContext:(CGLContextObj)ctx pixelFormat:(CGLPixelFormatObj)pf options:(NSDictionary*)dict
// ----------------------------------------------------------------------------
{
	CGColorSpaceRef working_colorspace = CGColorSpaceCreateDeviceRGB();
	CGColorSpaceRef output_colorspace = working_colorspace;
	
	NSMutableDictionary* mdict = [dict mutableCopy];
	[mdict setObject:(__bridge id)output_colorspace forKey:kCIContextOutputColorSpace];
	[mdict setObject:(__bridge id)working_colorspace forKey:kCIContextWorkingColorSpace];
	
	return [self real_contextWithCGLContext:ctx pixelFormat:pf options:mdict];
}
@end


@implementation GBGameDetailController

@synthesize gameName = _gameName;
@synthesize gameInfo = _gameInfo;
@synthesize gameScreenshot = _gameScreenshot;
@synthesize gameDetailLeftColumn = _gameDetailLeftColumn;
@synthesize currentItemGameId = _currentItemGameId;
@synthesize currentItemMusicPath = _currentItemMusicPath;
@synthesize currentItemGamePath = _currentItemGamePath;
@synthesize currentItemGameToRun = _currentItemGameToRun;
@synthesize screenShotIndex = _screenShotIndex;
@synthesize hasSingleValidSelection = _hasSingleValidSelection;


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[[GBPreferencesController sharedInstance] load];
	
	_screenShotIndex = 0;
	screenShotPaths = [[NSMutableArray alloc] init];
	
	_currentItemMusicPath = nil;
	_currentItemGamePath = nil;
	_currentItemGameToRun = nil;
	_currentItemGameId = nil;
	
	sequelGameId = 0;
	prequelGameId = 0;
	relatedGameId = 0;

	isShowingVersionInfo = NO;
	_hasSingleValidSelection = NO;
	isShowingDatabaseDownloadView = NO;

	[gameDetailView setWantsLayer:YES];
	[extrasView setWantsLayer:YES];
	[databaseDownloadView setWantsLayer:YES];
	
	[extrasView setFrame:[gameDetailView frame]];
	[[gameDetailView superview] addSubview:extrasView];
	[extrasView setHidden:YES];

	gameDetailView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
    extrasView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
	
	[backgroundView setHidden:NO];
	
	[prequelButton setEnabled:NO];
	[sequelButton setEnabled:NO];
	[relatedButton setEnabled:NO];
	[webPageButton setEnabled:NO];

	Method md1		= class_getClassMethod([CIContext class],@selector(contextWithCGLContext:pixelFormat:options:));
	Method md2		= class_getClassMethod([CIContext class],@selector(real_contextWithCGLContext:pixelFormat:options:));
	method_exchangeImplementations(md1,md2);
	 
	[self enablePalEffect:gPreferences.mPalEffectEnabled andEnableColorCorrection:gPreferences.mColorCorrectionEnabled];
	[palEffectButton setState:gPreferences.mPalEffectEnabled ? NSOnState : NSOffState];
	[colorCorrectionButton setState:gPreferences.mColorCorrectionEnabled ? NSOnState : NSOffState];
}
	 
	 
// ----------------------------------------------------------------------------
- (BOOL) validateMenuItem:(NSMenuItem*)item
// ----------------------------------------------------------------------------
{
	NSInteger tag = [item tag];

	if (tag == VALIDATION_TAG_GAME_INFO)
		return (isShowingVersionInfo || [gameDetailView isHidden]);

	if (tag == VALIDATION_TAG_VERSION_INFO)
		return (!isShowingVersionInfo || [gameDetailView isHidden]);

	if  (tag == VALIDATION_TAG_EXTRAS)
		return ![gameDetailView isHidden] && [extrasButton isEnabled];

	if  (tag == VALIDATION_TAG_NEXT_SCREENSHOT)
		return ![screenShotControl isHidden] && [screenShotControl isEnabledForSegment:0];

	if  (tag == VALIDATION_TAG_PREV_SCREENSHOT)
		return ![screenShotControl isHidden] && [screenShotControl isEnabledForSegment:1];

	if (tag == VALIDATION_TAG_FIND_VIDEO)
		return _hasSingleValidSelection;

	if (tag == VALIDATION_TAG_SHOW_WEBPAGE)
		return _hasSingleValidSelection;

	return YES;
}


// ----------------------------------------------------------------------------
- (void) showMultipleSelection
// ----------------------------------------------------------------------------
{
	self.gameName = @"Multiple Games Selected";
	self.gameInfo = @"";
	self.gameDetailLeftColumn = @"";
	self.gameScreenshot = [NSImage imageNamed:@"multiple"];
	self.currentItemGameId = nil;
	[extrasButton setEnabled:NO];
	[screenShotControl setHidden:YES];
	[screenShotLabel setHidden:YES];
	[palEffectButton setHidden:YES];
	[colorCorrectionButton setHidden:YES];
	[extrasDataSource.browserData removeAllObjects];
	[extrasDataSource.browserView reloadData];

	[extrasButton setTitle:@"Extras (0)"];
	[extrasButton setEnabled:NO];
	[prequelButton setEnabled:NO];
	[sequelButton setEnabled:NO];
	[relatedButton setEnabled:NO];
	[webPageButton setEnabled:NO];
	
	currentGameTableRow = nil;
	_hasSingleValidSelection = NO;
}


// ----------------------------------------------------------------------------
- (void) showEmptySelection
// ----------------------------------------------------------------------------
{
	self.gameName = @"Nothing Selected";
	self.gameInfo = @"";
	self.gameDetailLeftColumn = @"";
	self.gameScreenshot = nil;
	self.currentItemGameId = nil;
	[extrasButton setEnabled:NO];
	[screenShotControl setHidden:YES];
	[screenShotLabel setHidden:YES];
	[palEffectButton setHidden:YES];
	[colorCorrectionButton setHidden:YES];
	[extrasDataSource.browserData removeAllObjects];
	[extrasDataSource.browserView reloadData];

	[extrasButton setTitle:@"Extras (0)"];
	[extrasButton setEnabled:NO];
	[prequelButton setEnabled:NO];
	[sequelButton setEnabled:NO];
	[relatedButton setEnabled:NO];
	[webPageButton setEnabled:NO];
	
	currentGameTableRow = nil;
	_hasSingleValidSelection = NO;
}


// ----------------------------------------------------------------------------
- (void) hideAllFields
// ----------------------------------------------------------------------------
{
	self.gameName = @"";
	self.gameInfo = @"";
	self.gameDetailLeftColumn = @"";
	self.gameScreenshot = nil;
	self.currentItemGameId = nil;
	[extrasButton setEnabled:NO];
	[screenShotControl setHidden:YES];
	[screenShotLabel setHidden:YES];
	[palEffectButton setHidden:YES];
	[colorCorrectionButton setHidden:YES];
	[extrasDataSource.browserData removeAllObjects];
	[extrasDataSource.browserView reloadData];
	
	[extrasButton setTitle:@"Extras (0)"];
	[extrasButton setEnabled:NO];
	[prequelButton setEnabled:NO];
	[sequelButton setEnabled:NO];
	[relatedButton setEnabled:NO];
	[webPageButton setEnabled:NO];

	currentGameTableRow = nil;
	_hasSingleValidSelection = NO;
}


// ----------------------------------------------------------------------------
- (void) reloadGameDetails:(NSMutableDictionary*)selectedGame andUpdateExtrasBrowser:(BOOL)updateExtras
// ----------------------------------------------------------------------------
{
	if (selectedGame == nil)
		return;
	
	currentGameTableRow = selectedGame;
	NSInteger gameId = [[selectedGame objectForKey:@"GA_Id"] integerValue];
	gPreferences.mLastViewedGameId = gameId;

	BOOL reloadingSameGame = (gameId == [_currentItemGameId integerValue]);

	self.currentItemGameId = [NSNumber numberWithInteger:gameId];
	
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	SQLite3Connector* databaseConnector = gamebaseAppDelegate.databaseConnector;
	NSString* databasePath = gamebaseAppDelegate.databasePath;
	NSString* queryString = [NSString stringWithFormat:@"SELECT Games.*, Years.Year, Musicians.Musician, PGenres.ParentGenre, Genres.Genre, Publishers.Publisher, Programmers.Programmer, Languages.Language, Crackers.Cracker \
							 FROM Games INNER JOIN Years ON Games.YE_Id=Years.YE_Id JOIN Musicians ON Games.MU_id=Musicians.MU_Id JOIN Genres ON Games.GE_Id=Genres.GE_Id JOIN PGenres ON Genres.PG_Id=PGenres.PG_Id \
							 JOIN Publishers ON Games.PU_Id=Publishers.PU_Id JOIN Languages ON Games.LA_Id=Languages.LA_Id JOIN Programmers ON Games.PR_Id=Programmers.PR_Id JOIN Crackers ON Games.CR_Id=Crackers.CR_Id \
							 WHERE Games.GA_Id=%ld;", (long)gameId];
	NSMutableArray* queryResults = [databaseConnector getData:queryString fromDatabase:databasePath];
	if ([queryResults count] == 0)
	{
		[self hideAllFields];
		return;
	}
	
	NSMutableDictionary* gameDetails = [queryResults objectAtIndex:0];
	
	NSString* databaseStoragePath = nil;
	GBSourceListItem* databaseItem = [gamebaseAppDelegate.sourceListDataSource currentDatabaseItem];
	if (databaseItem != nil)
		databaseStoragePath = [databaseItem path];
	
	_hasSingleValidSelection = YES;
	
	self.gameName = [gameDetails objectForKey:@"Name"];
	
	if (isShowingVersionInfo)
	{
		NSString* crackersString = [gameDetails objectForKey:@"Cracker"];
		if (crackersString == nil)
			crackersString = @"(Unknown)";
		
		NSNumber* trainersNumber = [gameDetails objectForKey:@"V_Trainers"];
		NSString* trainersString = [trainersNumber integerValue] >= 0 ? [trainersNumber stringValue] : @"(Unknown)";
		
		NSString* featuresString = @"";
		if ([[gameDetails objectForKey:@"V_IncludedDocs"] integerValue] == 1)
			featuresString = [featuresString stringByAppendingString:@"Documentation"];
		
		if ([[gameDetails objectForKey:@"V_HighScoreSaver"] integerValue] == 1)
		{
			if ([featuresString length] > 0)
				featuresString = [featuresString stringByAppendingString:@", "];
				
			featuresString = [featuresString stringByAppendingString:@"Highscore-Saver"];
		}
		
		if ([[gameDetails objectForKey:@"V_LoadingScreen"] integerValue] == 1)
		{
			if ([featuresString length] > 0)
				featuresString = [featuresString stringByAppendingString:@", "];

			featuresString = [featuresString stringByAppendingString:@"Loading-Screen"];
		}

		if ([featuresString length] == 0)
			featuresString = @"None";
		
		NSString* palNtscString = @"";
		NSInteger palNtscValue = [[gameDetails objectForKey:@"V_PalNTSC"] integerValue];
		switch (palNtscValue)
		{
			case 0:
				palNtscString = @"PAL";
				break;
			case 1:
				palNtscString = @"PAL+NTSC";
				break;
			case 2:
				palNtscString = @"NTSC";
				break;
			case 3:
				palNtscString = @"PAL(+NTSC?)";
				break;
		}
		
		NSNumber* lengthNumber = [gameDetails objectForKey:@"V_Length"];
		NSString* lengthString = [NSString stringWithFormat:@"%ld Block%s", (long)[lengthNumber integerValue], [lengthNumber integerValue] == 1 ? "" : "s"];
		
		NSString* commentString = [gameDetails objectForKey:@"V_Comment"];
		if (commentString == nil)
			commentString = @"";
		
		self.gameDetailLeftColumn = @"Cracker:\nTrainers:\nExtra Features:\nPAL/NTSC:\nLength:\n";
		if ([commentString length] > 0)
			self.gameDetailLeftColumn = [_gameDetailLeftColumn stringByAppendingString:@"Comment:\n"];

		NSString* infoFormatString = @"%@\n%@\n%@\n%@\n%@\n%@\n";
		self.gameInfo = [NSString stringWithFormat:infoFormatString,
						 crackersString, 
						 trainersString,
						 featuresString, 
						 palNtscString, 
						 lengthString, 
						 commentString];
	}
	else
	{
		NSInteger year = [[gameDetails objectForKey:@"Year"] integerValue];
		NSString* yearString = (year > 9000) ? @"19??" : [NSString stringWithFormat:@"%ld", (long)year];
		
		NSInteger controlType = [[gameDetails objectForKey:@"Control"] integerValue];
		NSString* controlString = controlType == 2 ? @"Keyboard" : (controlType == 1 ? @"Joystick Port 1" : @"Joystick Port 2");
		NSInteger playersFrom = [[gameDetails objectForKey:@"PlayersFrom"] integerValue];
		NSInteger playersTo = [[gameDetails objectForKey:@"PlayersTo"] integerValue];
		BOOL playersSim = [[gameDetails objectForKey:@"PlayersSim"] integerValue] == 1;
		NSString* playersString = (playersFrom == playersTo) ? [NSString stringWithFormat:@"%ld Player%s Only", (long)playersFrom, playersFrom > 1 ? "s" : ""] : [NSString stringWithFormat:@"%ld - %ld %s", (long)playersFrom, (long)playersTo, playersSim ? "(Simultaneous)" : ""];
		if (playersFrom == -1 || playersTo == -1)
			playersString = @"(Unknown)";
		
		NSString* commentString = [gameDetails objectForKey:@"Comment"];
		if (commentString == nil)
			commentString = @"";
		
		self.gameDetailLeftColumn = @"Published:\nProgrammer:\nMusician:\nLanguage:\nGenre:\nPrimary Control:\nPlayers:\n";
		if ([commentString length] > 0)
			self.gameDetailLeftColumn = [_gameDetailLeftColumn stringByAppendingString:@"Comment:\n"];

		NSString* infoFormatString = @"%@ %@\n%@\n%@\n%@\n%@ - %@\n%@\n%@\n%@\n";
		self.gameInfo = [NSString stringWithFormat:infoFormatString, yearString,
						 [gameDetails objectForKey:@"Publisher"], 
						 [gameDetails objectForKey:@"Programmer"], 
						 [gameDetails objectForKey:@"Musician"],
						 [gameDetails objectForKey:@"Language"],
						 [gameDetails objectForKey:@"ParentGenre"],
						 [gameDetails objectForKey:@"Genre"],
						 controlString,
						 playersString,
						 commentString];
	}
	
	[screenShotPaths removeAllObjects];
	NSString* screenshotFilename = [gameDetails objectForKey:@"ScrnshotFilename"];
	if (databaseStoragePath != nil && [screenshotFilename length] > 0)
	{
		NSString* screenshotDirectoryName = @"Screenshots";
		NSString* screenshotForwardSlashed = [screenshotFilename stringByReplacingOccurrencesOfString:@"\\" withString:@"/"];
		NSString* relativeScreenshotPath = [screenshotDirectoryName stringByAppendingPathComponent:screenshotForwardSlashed];
		NSString* screenshotPath = [databaseStoragePath stringByAppendingPathComponent:relativeScreenshotPath];
		NSString* extension = [screenshotPath pathExtension];
		
		BOOL screenshotExists = YES;
		int index = 0;
		while (screenshotExists)
		{
			NSString* newScreenshotPath = index == 0 ? [screenshotPath stringByDeletingPathExtension] : [[screenshotPath stringByDeletingPathExtension] stringByAppendingFormat:@"_%d", index];
			newScreenshotPath = [newScreenshotPath stringByAppendingPathExtension:extension];
			BOOL isDir = NO;
			screenshotExists = [[NSFileManager defaultManager] fileExistsAtPath:newScreenshotPath isDirectory:&isDir];
			if (screenshotExists && !isDir)
				[screenShotPaths addObject:newScreenshotPath];
			
			index++;
		}
	}

	if (!reloadingSameGame)
	{
		_screenShotIndex = 0;
		
		int screenShotCount = [screenShotPaths count];
		BOOL visible = screenShotCount > 1;
		[screenShotControl setHidden:!visible];
		[screenShotLabel setHidden:!visible];
		[screenShotControl setEnabled:NO forSegment:0];
		[screenShotControl setEnabled:screenShotCount > 1 forSegment:1];
	
		if (screenShotCount > 0)
			[self setScreenShotWithIndex:0];
		else
			self.gameScreenshot = [NSImage imageNamed:@"noScreenshot"];
	}
	
	[palEffectButton setHidden:NO];
	[colorCorrectionButton setHidden:NO];
	
	NSString* sidFilename = [gameDetails objectForKey:@"SidFilename"];
	if ([sidFilename length] > 0)
	{
		NSString* musicDirectoryName = @"C64Music";
		NSString* musicNameForwardSlashed = [sidFilename stringByReplacingOccurrencesOfString:@"\\" withString:@"/"];
		NSString* relativeMusicPath = [musicDirectoryName stringByAppendingPathComponent:musicNameForwardSlashed];
		_currentItemMusicPath = [databaseStoragePath stringByAppendingPathComponent:relativeMusicPath];
		BOOL isDir = NO;
		BOOL musicExists = [[NSFileManager defaultManager] fileExistsAtPath:_currentItemMusicPath isDirectory:&isDir];
		if (!musicExists || isDir)
			_currentItemMusicPath = nil;
	}
	else
		_currentItemMusicPath = nil;
	
	_currentItemGamePath = [selectedGame objectForKey:@"FullGamePath"];
	
	NSString* gameToRun = [gameDetails objectForKey:@"FileToRun"];
	if ([gameToRun length] > 0)
		_currentItemGameToRun = gameToRun;
	else
		_currentItemGameToRun = nil;
	
	queryString = [NSString stringWithFormat:@"SELECT DisplayOrder, Name, Path FROM Extras WHERE GA_Id=%ld;", (long)gameId];
	NSMutableArray* gameExtras = [databaseConnector getData:queryString fromDatabase:databasePath];
	NSMutableArray* extraItems = [NSMutableArray arrayWithCapacity:[gameExtras count]];
	
	if (gameExtras != nil && [gameExtras count] > 0)
	{
		NSSortDescriptor* sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"DisplayOrder" ascending:YES];
		[gameExtras sortUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
		
		NSString* extraDirectoryPath = [databaseStoragePath stringByAppendingPathComponent:@"Extras"];
		
		for (NSMutableDictionary* gameExtraRow in gameExtras)
		{
			NSString* extraFilename = [gameExtraRow objectForKey:@"Path"];
			NSString* extraPath = @"";
			
			if ([extraFilename length] > 0)
			{
				NSString* extraFilenameForwardSlashed = [extraFilename stringByReplacingOccurrencesOfString:@"\\" withString:@"/"];
				extraPath = [extraDirectoryPath stringByAppendingPathComponent:extraFilenameForwardSlashed];
			}

			NSString* title = [extraPath lastPathComponent];
			NSString* subTitle = [gameExtraRow objectForKey:@"Name"];

			BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:extraPath isDirectory:NULL];
			if (exists)
			{
				GBExtrasBrowserItem* extrasBrowserItem = [[GBExtrasBrowserItem alloc] initWithPath:extraPath andTitle:title andSubTitle:subTitle];
				[extraItems addObject:extrasBrowserItem];
			}
		}
	}

	if (updateExtras)
	{
		extrasDataSource.browserData = extraItems;
		[extrasDataSource.browserView reloadData];
	}

	NSInteger availableExtras = [extrasDataSource.browserData count];
	[extrasButton setTitle:[NSString stringWithFormat:@"Extras (%ld)", (long)availableExtras]];
	[extrasButton setEnabled:availableExtras > 0];
	
	prequelGameId = [[gameDetails objectForKey:@"Prequel"] integerValue];
    NSMutableDictionary* gameRow = [gameTableDataSource findGameWithGameIdInAllGames:prequelGameId];
	[prequelButton setEnabled:gameRow != nil];

	sequelGameId = [[gameDetails objectForKey:@"Sequel"] integerValue];
    gameRow = [gameTableDataSource findGameWithGameIdInAllGames:sequelGameId];
	[sequelButton setEnabled:gameRow != nil];
	
	relatedGameId = [[gameDetails objectForKey:@"Related"] integerValue];
	gameRow = [gameTableDataSource findGameWithGameIdInAllGames:relatedGameId];
    [relatedButton setEnabled:gameRow != nil];
	
	[webPageButton setEnabled:YES];
}



// ----------------------------------------------------------------------------
- (void) setScreenShotWithIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	_screenShotIndex = index;
	int screenShotCount = [screenShotPaths count];
	NSImage* screenshot = [[NSImage alloc] initWithContentsOfFile:[screenShotPaths objectAtIndex:index]];
	[screenshot setSize:NSMakeSize(320.0f, 200.0f)];
	self.gameScreenshot = screenshot; 
	
	[screenShotControl setEnabled:_screenShotIndex > 0 forSegment:0];
	[screenShotControl setEnabled:_screenShotIndex < (screenShotCount - 1) forSegment:1];
	
	[screenShotLabel setStringValue:[NSString stringWithFormat:@"%ld of %d", _screenShotIndex + 1, screenShotCount]];
}


// ----------------------------------------------------------------------------
- (IBAction) previousScreenshot:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger newIndex = _screenShotIndex - 1;
	if (newIndex >= 0)
		[self setScreenShotWithIndex:newIndex];
}


// ----------------------------------------------------------------------------
- (IBAction) nextScreenshot:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger newIndex = _screenShotIndex + 1;
	if (newIndex < [screenShotPaths count])
		[self setScreenShotWithIndex:newIndex];
}


// ----------------------------------------------------------------------------
- (IBAction) changeScreenshot:(id)sender
// ----------------------------------------------------------------------------
{
	if ([sender isSelectedForSegment:0])
		[self previousScreenshot:self];
	else if ([sender isSelectedForSegment:1])
		[self nextScreenshot:self];
}


// ----------------------------------------------------------------------------
- (void) enablePalEffect:(BOOL)enablePalEffect andEnableColorCorrection:(BOOL)enableColorCorrection
// ----------------------------------------------------------------------------
{
	NSMutableArray* filters = [[NSMutableArray alloc] init];

	gPreferences.mPalEffectEnabled = enablePalEffect;
	gPreferences.mColorCorrectionEnabled = enableColorCorrection;
	
	if (enableColorCorrection)
	{
		[CICCS64ColorCorrectionFilter class];
		CIFilter* filter = [CIFilter filterWithName:@"CICCS64ColorCorrection"]; 
		[filter setDefaults]; 
		[filter setName:@"colorCorrection"]; 
		[filters addObject:filter];
	}

	if (enablePalEffect)
	{
		[CIPalEffectFilter class];
		CIFilter* filter = [CIFilter filterWithName:@"CIPalEffect"]; 
		[filter setDefaults]; 
		[filter setName:@"palFilter"]; 
		[filters addObject:filter];
	}	 
	
	if ([filters count] > 0)
		[screenShotView setContentFilters:filters];
	else
		[screenShotView setContentFilters:[NSArray array]];
}


// ----------------------------------------------------------------------------
- (IBAction) togglePalEffect:(id)sender
// ----------------------------------------------------------------------------
{
	[self enablePalEffect:[(NSButton*)sender state] == NSOnState andEnableColorCorrection:gPreferences.mColorCorrectionEnabled];
}


// ----------------------------------------------------------------------------
- (IBAction) toggleColorCorrection:(id)sender
// ----------------------------------------------------------------------------
{
	[self enablePalEffect:gPreferences.mPalEffectEnabled andEnableColorCorrection:[(NSButton*)sender state] == NSOnState];
}


// ----------------------------------------------------------------------------
- (IBAction) gameDetailsTypeChanged:(id)sender
// ----------------------------------------------------------------------------
{
	if ([sender tag] == 100)
		[self switchToGameInfoView:sender];
	else if ([sender tag] == 200)
		[self switchToVersionInfoView:sender];
}


// ----------------------------------------------------------------------------
- (IBAction) toggleExtras:(id)sender
// ----------------------------------------------------------------------------
{
	if ([extrasView isHidden])
		[self switchToExtrasView:sender];
	else
	{
		if (isShowingVersionInfo)
			[self switchToVersionInfoView:sender];
		else
			[self switchToGameInfoView:sender];
	}
}
	
	
// ----------------------------------------------------------------------------
- (IBAction) switchToGameInfoView:(id)sender
// ----------------------------------------------------------------------------
{
	// Set game details to show game info
	isShowingVersionInfo = NO;
	[self reloadGameDetails:currentGameTableRow andUpdateExtrasBrowser:NO];

	[gameInfoButton setState:NSOnState];
	
	[versionInfoButton setState:NSOffState];
	[extrasButton setState:NSOffState];

	if ([gameDetailView isHidden])
		[self triggerFlipTransition];
}


// ----------------------------------------------------------------------------
- (IBAction) switchToVersionInfoView:(id)sender
// ----------------------------------------------------------------------------
{
	// Set game details to show version info
	isShowingVersionInfo = YES;
	[self reloadGameDetails:currentGameTableRow andUpdateExtrasBrowser:NO];

	[versionInfoButton setState:NSOnState];

	[gameInfoButton setState:NSOffState];
	[extrasButton setState:NSOffState];
	
	if ([gameDetailView isHidden])
		[self triggerFlipTransition];
}


// ----------------------------------------------------------------------------
- (IBAction) switchToExtrasView:(id)sender
// ----------------------------------------------------------------------------
{
	if (![extrasButton isEnabled])
		return;
	
	[extrasDataSource.browserView setSelectionIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
	[extrasDataSource.browserView scrollIndexToVisible:0];

	[extrasButton setState:NSOnState];
	
	[versionInfoButton setState:NSOffState];
	[gameInfoButton setState:NSOffState];

	[self triggerFlipTransition];
}


// ----------------------------------------------------------------------------
- (void) triggerFlipTransition
// ----------------------------------------------------------------------------
{
#ifdef ROTATE_TRANSITION
    float flipDuration = [gameDetailView window].currentEvent.modifierFlags & NSShiftKeyMask ? 5.0f : 0.8f;
	
	//[gameDetailView setWantsLayer:YES];
	//[extrasView setWantsLayer:YES];
	
	[extrasButton setEnabled:NO];
	[gameInfoButton setEnabled:NO];
	[versionInfoButton setEnabled:NO];
	
	CATransform3D transform = CATransform3DIdentity;
	transform.m34 = 1.0 / -850;
	[gameDetailView.layer setTransform:transform];
	[extrasView.layer setTransform:transform];
	
    // The hidden layer is "in the back" and will be rotating forward. The visible layer is "in the front" and will be rotating backward
    CALayer* hiddenLayer = [(gameDetailView.isHidden ? gameDetailView : extrasView) layer];
    CALayer* visibleLayer = [(gameDetailView.isHidden ? extrasView : gameDetailView) layer];
    
    // Before we can "rotate" the window, we need to make the hidden view look like it's facing backward by rotating it pi radians (180 degrees). We make this its own transaction and supress animation, because this is already the assumed state
    [CATransaction begin];
    [CATransaction setValue:[NSNumber numberWithBool:YES] forKey:kCATransactionDisableActions];
    [hiddenLayer setValue:[NSNumber numberWithDouble:M_PI] forKeyPath:@"transform.rotation.y"];
    [CATransaction commit];
    
    // There's no way to know when we are halfway through the animation, so we have to use a timer. On a sufficiently fast machine (like a Mac) this is close enough. On something like an iPhone, this can cause minor drawing glitches
	NSTimer* timer = [NSTimer scheduledTimerWithTimeInterval:flipDuration/2.0 target:self selector:@selector(swapViews) userInfo:nil repeats:NO];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSEventTrackingRunLoopMode];
	
    // Both layers animate the same way, but in opposite directions (front to back versus back to front)
    [CATransaction begin];
    [hiddenLayer addAnimation:[self flipAnimationWithDuration:flipDuration isFront:NO] forKey:@"flipAnimation"];
    [visibleLayer addAnimation:[self flipAnimationWithDuration:flipDuration isFront:YES] forKey:@"flipAnimation"];
    [CATransaction commit];
    
    NSLog(@"Start");
#else
    
    [self swapViews];
    
#endif
}


// ----------------------------------------------------------------------------
- (CAAnimation*) flipAnimationWithDuration:(CGFloat)duration isFront:(BOOL)isFront
// ----------------------------------------------------------------------------
{
    // Rotating halfway (pi radians) around the Y axis gives the appearance of flipping
    CABasicAnimation* flipAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.y"];
	
    // The hidden view rotates from negative to make it look like it's in the back
    flipAnimation.toValue = [NSNumber numberWithDouble:!gameDetailView.isHidden ? (isFront ? -M_PI : M_PI) : (isFront ? M_PI : -M_PI)];
    
    // As the edge gets closer to us, it appears to move faster. Simulate this in 2D with an easing function
    flipAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
	
    // Set ourselves as the delegate so we can clean up when the animation is finished
    flipAnimation.delegate = isFront ? self : nil;
    flipAnimation.duration = duration;
	
    // Hold the view in the state reached by the animation until we can fix it, or else we get an annoying flicker
    flipAnimation.fillMode = kCAFillModeForwards;
    flipAnimation.removedOnCompletion = NO;
	
    return flipAnimation;
}


// ----------------------------------------------------------------------------
- (void) swapViews
// ----------------------------------------------------------------------------
{
    // At the point the window flips, change which view is visible, thus bringing it "to the front"
    [gameDetailView setHidden:![gameDetailView isHidden]];
    [extrasView setHidden:![extrasView isHidden]];

	[extrasDataSource.browserView reloadData];
	
	// This ugliness forces the browser to scroll by one pixel, otherwise it won't redraw
	// when we reload the data while it is hidden. 
	NSScrollView* scrollView = [extrasDataSource.browserView enclosingScrollView];
	float documentHeight = [[scrollView documentView] frame].size.height;
	float clipHeight = [[scrollView contentView] documentVisibleRect].size.height;
	float scrollPoint = documentHeight - clipHeight - 1.0f;
	[[scrollView contentView] scrollToPoint:NSMakePoint(0.0f, scrollPoint)];
}


// ----------------------------------------------------------------------------
- (void) animationDidStop:(CAAnimation*)animation finished:(BOOL)flag
// ----------------------------------------------------------------------------
{
    NSLog(@"Stop %@", animation);
    
    // Our animation is one-way and has been expended, so we can remove them
    [gameDetailView.layer removeAnimationForKey:@"flipAnimation"];
    [extrasView.layer removeAnimationForKey:@"flipAnimation"];
    
    // Although the "back" layer seemed to rotate forward, in reality it's still flipped.
    // Since this is already our assumed state, do not animate this
    [CATransaction begin];
    [CATransaction setValue:[NSNumber numberWithBool:YES] forKey:kCATransactionDisableActions];
    gameDetailView.layer.transform = CATransform3DIdentity;
    extrasView.layer.transform = CATransform3DIdentity;
    [CATransaction commit];
	
	NSInteger availableExtras = [extrasDataSource.browserData count];
	[extrasButton setEnabled:availableExtras > 0];

	[gameInfoButton setEnabled:YES];
	[versionInfoButton setEnabled:YES];

	//[gameDetailView setWantsLayer:NO];
	//[extrasView setWantsLayer:NO];
}


// ----------------------------------------------------------------------------
- (void) showDatabaseDownloadView
// ----------------------------------------------------------------------------
{
	[self hideAllFields];
	
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	GBSourceListItem* databaseItem = [gamebaseAppDelegate.sourceListDataSource currentDatabaseItem];
	if (databaseItem != nil)
		[databaseDownloadController prepareDownloadForDatabase:databaseItem];
	
	[databaseDownloadView setFrame:[gameDetailView frame]];
	[[gameDetailView superview] addSubview:databaseDownloadView];
	
	isShowingDatabaseDownloadView = YES;
}


// ----------------------------------------------------------------------------
- (void) hideDatabaseDownloadView
// ----------------------------------------------------------------------------
{
	[databaseDownloadView removeFromSuperview];
	isShowingDatabaseDownloadView = NO;
}


#pragma mark -
#pragma mark UI Actions


// ----------------------------------------------------------------------------
- (IBAction) findLongPlayVideo:(id)sender
// ----------------------------------------------------------------------------
{
	if (_gameName == nil)
		return;
	
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	[gamebaseAppDelegate findYoutubeLongplayForGame:_gameName];
}


// ----------------------------------------------------------------------------
- (IBAction) showSequel:(id)sender
// ----------------------------------------------------------------------------
{
	if (sequelGameId == 0)
		return;
	
	[gameTableDataSource selectGameWithGameId:sequelGameId];
}


// ----------------------------------------------------------------------------
- (IBAction) showPrequel:(id)sender
// ----------------------------------------------------------------------------
{
	if (prequelGameId == 0)
		return;
	
	[gameTableDataSource selectGameWithGameId:prequelGameId];
}


// ----------------------------------------------------------------------------
- (IBAction) showRelated:(id)sender
// ----------------------------------------------------------------------------
{
	if (relatedGameId == 0)
		return;
	
	[gameTableDataSource selectGameWithGameId:relatedGameId];
}


// ----------------------------------------------------------------------------
- (IBAction) showWebPage:(id)sender
// ----------------------------------------------------------------------------
{
	NSString* urlString = @"http://www.gamebase64.com/game.php?id=";
	urlString = [urlString stringByAppendingString:[_currentItemGameId stringValue]];
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:urlString]];
}


@end
