
#import <Cocoa/Cocoa.h>

@class GBGameDetailController;
@class GBImageBrowserDataSource;
@class GBGameCollection;


extern NSString* GBGamePBoardType;


enum GBColumnType
{
	COLUMN_CLASSIC = 0,
	COLUMN_HAS_SID_FILENAME,
	COLUMN_NAME,
	COLUMN_YEAR,
	COLUMN_PUBLISHER,
	COLUMN_GENRE,
	COLUMN_COMMENT,
	COLUMN_FILENAME,
	
	COLUMN_COUNT
};



@interface GBGameTableDataSource : NSObject
{
	IBOutlet NSTableView* _gameTableView;
	IBOutlet GBGameDetailController* gameDetailController;
	IBOutlet GBImageBrowserDataSource* imageBrowserDataSource;

	NSString* _databasePath;
	NSMutableArray* _gameTable;
	NSString* _gameTableStatusString;
	NSInteger _availableGameCount;

	GBGameCollection* _currentCollection;
	
	NSInteger savedGameId;
	
	NSArray* _draggedItems;

	IBOutlet NSSearchField* _toolbarSearchField;
	IBOutlet NSMenu* toolbarSearchFieldMenu;

	IBOutlet NSMenu* tableHeaderContextMenu;
	NSTableColumn* tableColumns[COLUMN_COUNT];

	NSMutableArray* _unfilteredGameTable;
	NSMutableArray* unfilteredSearchGameTable;
}

@property (nonatomic) NSString* databasePath;
@property (nonatomic) NSTableView* gameTableView;
@property (nonatomic) NSMutableArray* gameTable;
@property (nonatomic) NSMutableArray* unfilteredGameTable;
@property (nonatomic) NSString* gameTableStatusString;
@property (nonatomic) NSInteger availableGameCount;
@property (nonatomic) GBGameCollection* currentCollection;
@property (nonatomic) NSArray* draggedItems;
@property (nonatomic) NSSearchField* toolbarSearchField;


- (void) reloadGameDetailsFromRow:(NSInteger)rowIndex;
- (void) saveSelection;
- (void) restoreSelection;
- (void) selectAndShowIndex:(NSUInteger)index;

- (IBAction) selectRandomGame:(id)sender;

- (IBAction) searchStringEntered:(id)sender;
- (IBAction) searchTypeChanged:(id)sender;
- (void) adjustSearchTypeMenu;

- (void) sortUsingSortDescriptors;

- (void) filterGamesWithSearchString:(NSString*)searchString;
- (void) showUnfilteredSearchGameTable;

- (void) filterGamesFromCollection:(GBGameCollection*)collection keepSelection:(BOOL)keepSelection;
- (void) showUnfilteredGameTable;

- (void) clearTable;
- (void) updateSelection;
- (IBAction) deleteSelectedItems:(id)sender;
   
- (NSInteger) findRowIndexOfGameWithGameId:(NSInteger)gameId;
- (NSMutableDictionary*) findGameWithGameIdInAllGames:(NSInteger)gameId;
- (void) selectGameWithGameId:(NSInteger)gameId;

- (IBAction) tableColumnStatusChanged:(id)sender;

@end


@interface GBBrowserView : NSTableView
{

}

@end

