
#import "GBGameTableDataSource.h"
#import "GBSourceListDataSource.h"
#import "GameBaseAppDelegate.h"
#import "GBPreferencesController.h"
#import "GBGameDetailController.h"
#import "GBImageBrowserDataSource.h"
#import "GBGameCollection.h"


@implementation GBGameTableDataSource

static NSImage* sGBStarImage = nil;
static NSImage* sGBStarWhiteImage = nil;
static NSImage* sGBMusicImage = nil;
static NSImage* sGBMusicWhiteImage = nil;


@synthesize databasePath = _databasePath;
@synthesize gameTableView = _gameTableView;
@synthesize gameTable = _gameTable;
@synthesize unfilteredGameTable = _unfilteredGameTable;
@synthesize gameTableStatusString = _gameTableStatusString;
@synthesize availableGameCount = _availableGambeCount;
@synthesize currentCollection = _currentCollection;
@synthesize draggedItems = _draggedItems;
@synthesize toolbarSearchField = _toolbarSearchField;


NSString* GBGamePBoardType = @"GBGamePBoardType";


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		_databasePath = nil;
		_gameTable = nil;
		_unfilteredGameTable = nil;
		unfilteredSearchGameTable = nil;
		_draggedItems = nil;
		_currentCollection = nil;
	}
	return self;
}


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[[GBPreferencesController sharedInstance] load];
	
	sGBStarImage = [NSImage imageNamed:@"star_filled"];
    sGBStarWhiteImage = [NSImage imageNamed:@"star_white_filled"];
	sGBMusicImage = [NSImage imageNamed:@"music_filled"];
    sGBMusicWhiteImage = [NSImage imageNamed:@"music_white_filled"];
    
	savedGameId = -1;
	
	[self adjustSearchTypeMenu];

    // Set star image on classic column header
	NSArray* columns = [_gameTableView tableColumns];
	for (NSTableColumn* column in columns)
	{
		NSString* identifier = [column identifier];
		if ([identifier isEqualToString:@"Classic"])
			tableColumns[COLUMN_CLASSIC] = column;
		else if ([identifier isEqualToString:@"SidFilename"])
			tableColumns[COLUMN_HAS_SID_FILENAME] = column;
		else if ([identifier isEqualToString:@"Name"])
			tableColumns[COLUMN_NAME] = column;
		else if ([identifier isEqualToString:@"Year"])
			tableColumns[COLUMN_YEAR] = column;
		else if ([identifier isEqualToString:@"Publisher"])
			tableColumns[COLUMN_PUBLISHER] = column;
		else if ([identifier isEqualToString:@"Genre"])
			tableColumns[COLUMN_GENRE] = column;
		else if ([identifier isEqualToString:@"Comment"])
			tableColumns[COLUMN_COMMENT] = column;
		else if ([identifier isEqualToString:@"Filename"])
			tableColumns[COLUMN_FILENAME] = column;
	}

	[[tableColumns[COLUMN_CLASSIC] headerCell] setImage:[NSImage imageNamed:@"star"]];
	[[tableColumns[COLUMN_HAS_SID_FILENAME] headerCell] setImage:[NSImage imageNamed:@"music"]];
	
	NSArray* items = [tableHeaderContextMenu itemArray];
	for (NSMenuItem* item in items)
	{
		GBColumnType columnType = (GBColumnType)[item tag];
		NSTableColumn* column = tableColumns[columnType];
		if ([column isHidden])
			[item setState:NSOffState];
		else
			[item setState:NSOnState];
	}
	
	NSSortDescriptor* sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"Name" ascending:YES selector:@selector(localizedCaseInsensitiveCompare:)];
	[_gameTableView setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	
	[[_gameTableView headerView] setMenu:tableHeaderContextMenu];
    
//    float extraHeaderSize = 10.0f;
//    
//    for (NSView* subview in [_scrollView subviews])
//    {
//        for(NSView* subSubView in [subview subviews])
//        {
//            if([[subSubView className] isEqualToString:@"NSTableHeaderView"] && [[subview className] isEqualToString:@"NSClipView"])
//            {
//                [subSubView setFrameSize:NSMakeSize(subSubView.frame.size.width, subSubView.frame.size.height+extraHeaderSize)];//HeaderView Frame
//                [subview setFrameSize:NSMakeSize(subview.frame.size.width, subview.frame.size.height+extraHeaderSize)];//ClipView Frame
//            }
//            
//        }
//        if ([[subview className] isEqualToString:@"_NSCornerView"])
//        {
//            [subview setFrameSize:NSMakeSize(subview.frame.size.width, subview.frame.size.height+extraHeaderSize)]; //CornerView Frame
//        }
//    }
}



// ----------------------------------------------------------------------------
- (void) sortUsingSortDescriptors
// ----------------------------------------------------------------------------
{
	[_gameTable sortUsingDescriptors:[_gameTableView sortDescriptors]];
	[_gameTableView reloadData];
}


// ----------------------------------------------------------------------------
- (void) saveSelection
// ----------------------------------------------------------------------------
{
	NSIndexSet* selectedIndices = [_gameTableView selectedRowIndexes];
	NSUInteger selectedRow = [selectedIndices firstIndex];
	if (selectedRow != NSNotFound)
	{
		NSMutableDictionary* selectedGame = [_gameTable objectAtIndex:selectedRow];
		savedGameId = [[selectedGame objectForKey:@"GA_Id"] integerValue];
		[_gameTableView deselectRow:selectedRow];
	}
}


// ----------------------------------------------------------------------------
- (void) restoreSelection
// ----------------------------------------------------------------------------
{
	if (savedGameId == -1)
		return;
		
	NSInteger rowIndex = [self findRowIndexOfGameWithGameId:savedGameId];
	
	[self selectAndShowIndex:rowIndex];
	
	savedGameId = -1;
}


// ----------------------------------------------------------------------------
- (void) selectAndShowIndex:(NSUInteger)index 
// ----------------------------------------------------------------------------
{
	[_gameTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:index] byExtendingSelection:NO];
	[_gameTableView scrollRowToVisible:index];
	[[imageBrowserDataSource browserView] setSelectionIndexes:[NSIndexSet indexSetWithIndex:index] byExtendingSelection:NO];
	[[imageBrowserDataSource browserView] scrollIndexToVisible:index];
}


// ----------------------------------------------------------------------------
- (NSInteger) findRowIndexOfGameWithGameId:(NSInteger)gameId
// ----------------------------------------------------------------------------
{
	if (gameId == 0)
		return -1;
	
	NSInteger rowIndex = 0;
	for (NSMutableDictionary* gameTableRow in _gameTable)
	{
		if ([[gameTableRow objectForKey:@"GA_Id"] integerValue] == gameId)
			return rowIndex;
			
		rowIndex++;
	}
	
	return -1;
}


// ----------------------------------------------------------------------------
- (void) selectAndShowGameWithId:(NSInteger)gameId
// ----------------------------------------------------------------------------
{
    [self showUnfilteredGameTable];
    
    GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
    GBSourceListDataSource* sourceListDataSource = gamebaseAppDelegate.sourceListDataSource;
    [sourceListDataSource selectCurrentDatabaseItem];
    
    //NSMutableDictionary* gameRow = [self findGameWithGameIdInAllGames:gameId];
    
    NSInteger rowIndex = [self findRowIndexOfGameWithGameId:gameId];
    if (rowIndex != -1)
        [self selectAndShowIndex:rowIndex];
}



// ----------------------------------------------------------------------------
- (NSMutableDictionary*) findGameWithGameIdInAllGames:(NSInteger)gameId
// ----------------------------------------------------------------------------
{
    if (gameId == 0)
        return nil;
    
    for (NSMutableDictionary* gameTableRow in _unfilteredGameTable)
    {
        if ([[gameTableRow objectForKey:@"GA_Id"] integerValue] == gameId)
            return gameTableRow;
    }

    return nil;
}



// ----------------------------------------------------------------------------
- (void) selectGameWithGameId:(NSInteger)gameId
// ----------------------------------------------------------------------------
{
	NSInteger row = [self findRowIndexOfGameWithGameId:gameId];
	if (row != -1)
		[self selectAndShowIndex:row];
    else
        [self selectAndShowGameWithId:gameId];
}


// ----------------------------------------------------------------------------
- (IBAction) selectRandomGame:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger randomRowIndex = random() % [_gameTable count];
	[self selectAndShowIndex:randomRowIndex];
}



// ----------------------------------------------------------------------------
- (void) reloadGameDetailsFromRow:(NSInteger)rowIndex
// ----------------------------------------------------------------------------
{
	[gameDetailController reloadGameDetails:[_gameTable objectAtIndex:rowIndex] andUpdateExtrasBrowser:YES];
}


// ----------------------------------------------------------------------------
- (void) filterGamesWithSearchString:(NSString*)searchString
// ----------------------------------------------------------------------------
{
	NSMutableArray* gameTableRowsMatchingSearchString = [NSMutableArray arrayWithCapacity:[_gameTable count]];
	
	NSMutableArray* imageBrowserData = imageBrowserDataSource.browserData;
	NSMutableArray* imageBrowserDataMatchingSearchString = [NSMutableArray arrayWithCapacity:[imageBrowserData count]];
	
	if (unfilteredSearchGameTable == nil)
	{
		unfilteredSearchGameTable = [_gameTable mutableCopy];
		imageBrowserDataSource.unfilteredSearchBrowserData = [imageBrowserData mutableCopy]; 
	}
	
	if ([searchString length] < 2)
	{
		[self showUnfilteredSearchGameTable];
		return;
	}
	
	NSRange range;
	BOOL match = NO;
	NSInteger index = 0;
	for (NSMutableDictionary* gameTableRow in unfilteredSearchGameTable)
	{
		match = NO;
		switch (gPreferences.mSearchType)
		{
			case SEARCH_ALL:
				range = [[gameTableRow objectForKey:@"Name"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
				match = range.location != NSNotFound;
				if (!match)
				{
					range = [[gameTableRow objectForKey:@"Publisher"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
					match = range.location != NSNotFound;
					if (!match)
					{
						range = [[gameTableRow objectForKey:@"Genre"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
						match = range.location != NSNotFound;
						if (!match)
						{	
							range = [[gameTableRow objectForKey:@"Year"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
							match = range.location != NSNotFound;
						}
					}
				}
				break;
			case SEARCH_NAME:
				range = [[gameTableRow objectForKey:@"Name"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
				match = range.location != NSNotFound;
				break;
				
			case SEARCH_YEAR:
				range = [[gameTableRow objectForKey:@"Year"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
				match = range.location != NSNotFound;
				break;
				
			case SEARCH_GENRE:
				range = [[gameTableRow objectForKey:@"Genre"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
				match = range.location != NSNotFound;
				break;
				
			case SEARCH_PUBLISHER:
				range = [[gameTableRow objectForKey:@"Publisher"] rangeOfString:searchString options:NSCaseInsensitiveSearch];
				match = range.location != NSNotFound;
				break;
		}
		
		if (match)
		{
			[gameTableRowsMatchingSearchString addObject:gameTableRow];
			[imageBrowserDataMatchingSearchString addObject:[imageBrowserDataSource.unfilteredSearchBrowserData objectAtIndex:index]];
		}
		
		index++;
	}
	
	_gameTable = gameTableRowsMatchingSearchString;
	[_gameTableView reloadData];
	
	imageBrowserDataSource.browserData = imageBrowserDataMatchingSearchString;
	[imageBrowserDataSource reloadData];
	
	if ([gameTableRowsMatchingSearchString count] > 0)
	{
		[self selectAndShowIndex:0];
		[self reloadGameDetailsFromRow:0];
	}
	else
	{
		[gameDetailController showEmptySelection];
	}
}


// ----------------------------------------------------------------------------
- (void) showUnfilteredSearchGameTable
// ----------------------------------------------------------------------------
{
	if (unfilteredSearchGameTable == nil)
		return;
	
	[self saveSelection];
	
	self.gameTable = unfilteredSearchGameTable;
	imageBrowserDataSource.browserData = imageBrowserDataSource.unfilteredSearchBrowserData;
	
	unfilteredSearchGameTable = nil;
	imageBrowserDataSource.unfilteredSearchBrowserData = nil;
	
	[_gameTableView reloadData];
	[imageBrowserDataSource reloadData];
	
	[self restoreSelection];
}


// ----------------------------------------------------------------------------
- (void) filterGamesFromCollection:(GBGameCollection*)collection keepSelection:(BOOL)keepSelection
// ----------------------------------------------------------------------------
{
	if (collection == nil)
		return;
	
	[_toolbarSearchField setStringValue:@""];
	
	_currentCollection = collection;
	
	NSMutableArray* gameIds = collection.gameIds;
	NSMutableArray* gameTableRowsMatchingGameIds = [NSMutableArray arrayWithCapacity:[gameIds count]];
	NSMutableArray* imageBrowserDataMatchingGameIds = [NSMutableArray arrayWithCapacity:[gameIds count]];
	
	// Convert the game ids we want to filter first from NSNumbers to integers
	// so that the inner compare loop doesn't require to send messages
	int filteredGameIdsCount = [gameIds count];
	int* filteredGameIdValues = new int[filteredGameIdsCount];
	int index = 0;
	for (NSNumber* filteredGameId in gameIds)
	{
		filteredGameIdValues[index] = [filteredGameId intValue];
		index++;
	}
	
	index = 0;
	for (NSMutableDictionary* gameTableRow in _unfilteredGameTable)
	{
		NSNumber* gameId = [gameTableRow objectForKey:@"GA_Id"];
		int gameIdValue = [gameId intValue];
		
		for (int i = 0; i < filteredGameIdsCount; i++)
		{
			if (filteredGameIdValues[i] == gameIdValue)
			{
				[gameTableRowsMatchingGameIds addObject:gameTableRow];
				[imageBrowserDataMatchingGameIds addObject:[imageBrowserDataSource.unfilteredBrowserData objectAtIndex:index]];
			}
		}
		
		index++;
	}
	
    delete[] filteredGameIdValues;
    
	self.gameTable = gameTableRowsMatchingGameIds;
	[self sortUsingSortDescriptors];
	
	imageBrowserDataSource.browserData = imageBrowserDataMatchingGameIds;
	[imageBrowserDataSource reloadData];
	
	if ([gameTableRowsMatchingGameIds count] > 0)
	{
        if (!keepSelection)
        {
            [self selectAndShowIndex:0];
            [self reloadGameDetailsFromRow:0];
        }
	}
	else
		[gameDetailController showEmptySelection];
}


// ----------------------------------------------------------------------------
- (void) showUnfilteredGameTable
// ----------------------------------------------------------------------------
{
	if (_unfilteredGameTable == nil)
		return;
	
	[self saveSelection];
	
	self.gameTable = [_unfilteredGameTable mutableCopy];
    imageBrowserDataSource.browserData = [imageBrowserDataSource.unfilteredBrowserData mutableCopy];
	
	//imageBrowserDataSource.unfilteredBrowserData = nil;
	
	[_gameTableView reloadData];
	[imageBrowserDataSource reloadData];
	
	[self restoreSelection];
	
    [_toolbarSearchField setStringValue:@""];
    
	_currentCollection = nil;
}


// ----------------------------------------------------------------------------
- (void) clearTable
// ----------------------------------------------------------------------------
{
	[gameDetailController hideAllFields];
	
	[_gameTable removeAllObjects];
	[imageBrowserDataSource.browserData removeAllObjects];
	
	[_gameTableView reloadData];
	[imageBrowserDataSource reloadData];
	
	//imageBrowserDataSource.unfilteredBrowserData = nil;
	
	self.availableGameCount = 0;
	
	[_toolbarSearchField setStringValue:@""];
}


// ----------------------------------------------------------------------------
- (void) updateSelection
// ----------------------------------------------------------------------------
{
    NSIndexSet* selectedIndices = [_gameTableView selectedRowIndexes];
    
    if ([selectedIndices count] > 1)
    {
        [gameDetailController showMultipleSelection];
        return;
    }
    else if ([selectedIndices count] == 0)
    {
        [gameDetailController showEmptySelection];
        return;
    }
    
    NSUInteger selectedRow = [selectedIndices firstIndex];
    if (selectedRow != NSNotFound)
    {
        GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
        [gamebaseAppDelegate.loadingProgressIndicator setHidden:YES];
        
        [self reloadGameDetailsFromRow:selectedRow];
    }
}


// ----------------------------------------------------------------------------
- (IBAction) deleteSelectedItems:(id)sender
// ----------------------------------------------------------------------------
{
	if (_currentCollection == nil)
		return;

	NSMutableArray* gameIds = _currentCollection.gameIds;
	
	NSIndexSet* selectedRowIndices = [_gameTableView selectedRowIndexes];
	NSMutableIndexSet* selectedCollectionIndices = [[NSMutableIndexSet alloc] init];

	NSUInteger index = [selectedRowIndices firstIndex];
	NSUInteger newSelectionIndex = index;
	while (index != NSNotFound)
	{
		[_gameTableView deselectRow:index];
		
		NSMutableDictionary* gameTableRow = [_gameTable objectAtIndex:index];
		NSNumber* gameId = [gameTableRow objectForKey:@"GA_Id"];
	
		NSInteger collectionIndex = 0;
		for (NSNumber* gameIdNumber in gameIds)
		{
			if ([gameIdNumber isEqualToNumber:gameId])
				[selectedCollectionIndices addIndex:collectionIndex];
				
			collectionIndex++;
		}
		
		index = [selectedRowIndices indexGreaterThanIndex:index];
	}
	
	[gameIds removeObjectsAtIndexes:selectedCollectionIndices];
	[_currentCollection saveToFile];
    [self filterGamesFromCollection:_currentCollection keepSelection:NO];
	[self selectAndShowIndex:newSelectionIndex];
	
}


// ----------------------------------------------------------------------------
- (IBAction) tableColumnStatusChanged:(id)sender
// ----------------------------------------------------------------------------
{
    if (sender == nil)
		return;

	NSMenuItem* menuItem = (NSMenuItem*) sender;
	
	GBColumnType columnType = (GBColumnType) [sender tag];
	NSTableColumn* column = tableColumns[columnType];
	
	if ([menuItem state] == NSOffState)
	{
		[menuItem setState:NSOnState];
		[column setHidden:NO];
	}
	else if ([menuItem state] == NSOnState)
	{
		[menuItem setState:NSOffState];
		[column setHidden:YES];
	}
}




#pragma mark -
#pragma mark data source methods

// ----------------------------------------------------------------------------
- (NSInteger)numberOfRowsInTableView:(NSTableView*)tableView
// ----------------------------------------------------------------------------
{
    return [_gameTable count];
}


// ----------------------------------------------------------------------------
- (id)tableView:(NSTableView*)tableView objectValueForTableColumn:(NSTableColumn*)tableColumn row:(NSInteger)index
// ----------------------------------------------------------------------------
{
	id object = [[_gameTable objectAtIndex:index] objectForKey:[tableColumn identifier]];
    BOOL selected = [tableView isRowSelected:index];
    
	if ([[tableColumn identifier] isEqualToString:@"Classic"])
	{
		BOOL is_classic = [object intValue];
        return is_classic ? selected ? sGBStarWhiteImage : sGBStarImage : nil;
	}
	else if ([[tableColumn identifier] isEqualToString:@"SidFilename"])
	{
		BOOL has_music = [object length] > 0;
        return has_music ? selected ? sGBMusicWhiteImage : sGBMusicImage : nil;
	}
	
	return object;
}


#if 0

// ----------------------------------------------------------------------------
- (void) tableView:(NSTableView *)tableView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)index
// ----------------------------------------------------------------------------
{
    NSLog(@"edit column %@, object: %@", tableColumn, object);
}

#endif

// ----------------------------------------------------------------------------
- (void)tableView:(NSTableView*)tableView sortDescriptorsDidChange:(NSArray*)oldDescriptors
// ----------------------------------------------------------------------------
{
	[self saveSelection];
	
	[_gameTable sortUsingDescriptors:[tableView sortDescriptors]];
	[tableView reloadData];

	[self restoreSelection];
}


// ----------------------------------------------------------------------------
- (BOOL) tableView:(NSTableView*)tableView writeRowsWithIndexes:(NSIndexSet*)rowIndexes toPasteboard:(NSPasteboard*)pasteboard
// ----------------------------------------------------------------------------
{
	NSMutableArray* gameIds = [[NSMutableArray alloc] init];
	NSString* firstItemGamePath = nil;

	NSUInteger index = [rowIndexes firstIndex];
	while (index != NSNotFound)
	{
		NSMutableDictionary* gameRow = [_gameTable objectAtIndex:index];
		[gameIds addObject:[gameRow objectForKey:@"GA_Id"]];
		
		if (firstItemGamePath == nil)
			firstItemGamePath = [gameRow objectForKey:@"FullGamePath"];
		
		index = [rowIndexes indexGreaterThanIndex:index];
	}
	
	_draggedItems = gameIds;
	
    // Provide data for our custom type, and simple NSStrings.
    [pasteboard declareTypes:[NSArray arrayWithObjects:GBGamePBoardType, NSStringPboardType, nil] owner:self];
    [pasteboard setData:[NSData data] forType:GBGamePBoardType];
	
	if (firstItemGamePath != nil)
		[pasteboard setString:firstItemGamePath forType:NSStringPboardType];
	
    return YES;
}


#pragma mark -
#pragma mark delegate methods

// ----------------------------------------------------------------------------
- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(NSInteger)rowIndex
// ----------------------------------------------------------------------------
{
    GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	return gamebaseAppDelegate.gameTableLoaded;
}


// ----------------------------------------------------------------------------
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
// ----------------------------------------------------------------------------
{
    [self updateSelection];
}


// ----------------------------------------------------------------------------
- (NSString *)tableView:(NSTableView*)tableView typeSelectStringForTableColumn:(NSTableColumn*)tableColumn row:(NSInteger)row
// ----------------------------------------------------------------------------
{
	NSString* identifier = [tableColumn identifier];
	if ([identifier isEqualToString:@"Name"])
		return [[_gameTable objectAtIndex:row] objectForKey:@"Name"];
	else
		return @"";
}


// ----------------------------------------------------------------------------
- (BOOL) tableView:(NSTableView *)tableView shouldEditTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)rowIndex
// ----------------------------------------------------------------------------
{
    return NO;
    //return [[tableColumn identifier] isEqualToString:@"Classic"];
}


// ----------------------------------------------------------------------------
- (void)tableView:(NSTableView *)tableView didClickRow:(NSInteger)rowIndex andColumn:(NSInteger)columnIndex
// ----------------------------------------------------------------------------
{
    NSTableColumn* column = [tableView.tableColumns objectAtIndex:columnIndex];
    
    if (column != nil && [column.identifier isEqualToString:@"Classic"])
    {
        NSMutableDictionary* gameRow = [_gameTable objectAtIndex:rowIndex];
        NSNumber* number = [gameRow objectForKey:@"Classic"];
        BOOL new_value = ![number intValue];
        NSNumber* new_number = [NSNumber numberWithInt:new_value];
        [gameRow setObject:new_number forKey:@"Classic"];
        
        NSNumber* gameId = [gameRow objectForKey:@"GA_Id"];
        NSString* query = [NSString stringWithFormat:@"UPDATE Games SET Classic=%d WHERE Games.GA_Id=%ld;", new_value, [gameId integerValue]];
        GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
        SQLite3Connector* databaseConnector = gamebaseAppDelegate.databaseConnector;
        NSString* database = gamebaseAppDelegate.databasePath;
        
        BOOL result = [databaseConnector executeQuery:query inDatabase:database];
        
        if (!result)
            NSLog(@"Database update failed!");
        
        GBSourceListDataSource* sourceListDataSource = gamebaseAppDelegate.sourceListDataSource;
        if (new_value)
            [sourceListDataSource addGameIdToClassicsCollection:gameId];
        else
            [sourceListDataSource removeGameIdFromClassicsCollection:gameId];

        if (_currentCollection != nil)
            [self filterGamesFromCollection:_currentCollection keepSelection:YES];
        else
            [tableView reloadData];
        
        [self updateSelection];
    }
}


#pragma mark -
#pragma mark search support


// ----------------------------------------------------------------------------
- (IBAction) searchStringEntered:(id)sender
// ----------------------------------------------------------------------------
{
	[self filterGamesWithSearchString:[sender stringValue]];
}


// ----------------------------------------------------------------------------
- (IBAction) searchTypeChanged:(id)sender
// ----------------------------------------------------------------------------
{
	gPreferences.mSearchType = (GBSearchType) [sender tag];
	[self searchStringEntered:_toolbarSearchField];
	[self adjustSearchTypeMenu];
}


// ----------------------------------------------------------------------------
- (void) adjustSearchTypeMenu
// ----------------------------------------------------------------------------
{
	// set the search field menu to show the currently selected search type
	NSArray* menuItems = [toolbarSearchFieldMenu itemArray];
	for (id menuItem in menuItems)
	{
		[menuItem setState:NSOffState];
		
		if ([menuItem tag] == (NSInteger)gPreferences.mSearchType)
		{
			[menuItem setState:NSOnState];
			if (gPreferences.mSearchType == SEARCH_ALL)
				[[_toolbarSearchField cell] setPlaceholderString:@"Search"];
			else
				[[_toolbarSearchField cell] setPlaceholderString:[NSString stringWithFormat:@"%@ Search", [menuItem title]]];
		}
	}
	
	[[_toolbarSearchField cell] setSearchMenuTemplate:toolbarSearchFieldMenu];
}



@end


#pragma mark -


@implementation GBBrowserView


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[self setDoubleAction:@selector(doubleClick:)];
}


// ----------------------------------------------------------------------------
- (void) doubleClick:(id)sender
// ----------------------------------------------------------------------------
{	
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	[gamebaseAppDelegate playGame:self];
}


// ----------------------------------------------------------------------------
- (NSMenu*) menuForEvent:(NSEvent *)event
// ----------------------------------------------------------------------------
{
	NSPoint position = [self convertPoint:[event locationInWindow] fromView:nil];
	NSInteger row = [self rowAtPoint:position];
	if (row == -1)
        return nil;
    
	NSIndexSet* set = [self selectedRowIndexes];
	if (![set containsIndex:row])
		[self selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];

	if ([set count] > 1)
		return nil;
	
    BOOL isCollection = [(GBGameTableDataSource*)self.dataSource currentCollection] != nil;
    BOOL isSmartCollection = [[(GBGameTableDataSource*)self.dataSource currentCollection] isSmart];
    BOOL hideMenuItem = (!isCollection) || isSmartCollection;
    NSMenuItem* menuItem = [self.menu itemWithTag:1100];
    if (menuItem != nil)
        [menuItem setHidden:hideMenuItem];

    menuItem = [self.menu itemWithTag:1101];
    if (menuItem != nil)
        [menuItem setHidden:hideMenuItem];
    
	return [super menuForEvent:event];
}


// ----------------------------------------------------------------------------
- (void) keyDown:(NSEvent*)event
// ----------------------------------------------------------------------------
{
	NSString* characters = [event charactersIgnoringModifiers];
	unichar character = [characters characterAtIndex:0];
	
	if (character == '\r' || character == 3)
	{
		NSIndexSet* set = [self selectedRowIndexes];
		if ([set count] == 1)
		{
			GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
			[gamebaseAppDelegate playGame:self];
			return;
		}
	}
	else if (character == 0xf728 || character == 0x007f)
	{
		GBGameTableDataSource* tableDataSource = (GBGameTableDataSource*)[self dataSource];
		[tableDataSource deleteSelectedItems:self];
		return;
	}
	
	[super keyDown:event];
}


// ----------------------------------------------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
// ----------------------------------------------------------------------------
{
    NSPoint globalLocation = [theEvent locationInWindow];
    NSPoint localLocation = [self convertPoint:globalLocation fromView:nil];
    NSInteger clickedRow = [self rowAtPoint:localLocation];
    NSInteger clickedColumn = [self columnAtPoint:localLocation];
    
    [super mouseDown:theEvent];
    
    if (clickedRow != -1 && clickedColumn != -1)
        [(GBGameTableDataSource*)self.delegate tableView:self didClickRow:clickedRow andColumn:clickedColumn];
}


// ----------------------------------------------------------------------------
- (NSImage*) dragImageForRowsWithIndexes:(NSIndexSet*)dragRows tableColumns:(NSArray*)tableColumns event:(NSEvent*)dragEvent offset:(NSPointPointer)dragImageOffset
// ----------------------------------------------------------------------------
{
	if ([dragRows count] == 1)
	{
		NSArray* columns = [self tableColumns];
		NSImage* defaultImage = [super dragImageForRowsWithIndexes:dragRows tableColumns:columns event:dragEvent offset:dragImageOffset];
		NSSize imageSize = [defaultImage size];
		NSImage* image = [[NSImage alloc] initWithSize:imageSize];
		[image lockFocus];
		NSBezierPath* path = [NSBezierPath bezierPathWithRect:NSMakeRect(0.0f, 0.0f, imageSize.width, imageSize.height)];
		[[NSColor colorWithCalibratedWhite:1.0f alpha:0.5f] set];
		[path fill];
		[[NSColor colorWithCalibratedWhite:0.0f alpha:0.5f] set];
		[path stroke];
		[defaultImage drawAtPoint:NSMakePoint(0, 0) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0f];
		[image unlockFocus];
		
		return image;
	}
	else if ([dragRows count] < 10)
	{
		NSImage* iconImage = [NSImage imageNamed:@"NSMultipleDocuments"];
		NSImage* badgeImage = [NSImage imageNamed:@"dragBadge"]; 
		
		NSRect iconImageRect = NSMakeRect(0.0f, 0.0f, [iconImage size].width, [iconImage size].height);
		NSImage* image = [[NSImage alloc] initWithSize:NSMakeSize(64.0f, 64.0f)];
		[image lockFocus];
		[iconImage drawInRect:NSMakeRect(16.0f, 16.0f, 32.0f, 32.0f) fromRect:iconImageRect operation:NSCompositeSourceOver fraction:0.5f];
		[badgeImage drawAtPoint:NSMakePoint(38.0f, 10.0f) fromRect:NSZeroRect operation:NSCompositeSourceOver fraction:1.0f];
		NSDictionary* normalAttrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont boldSystemFontOfSize:11.0f], NSFontAttributeName,
									 [NSColor whiteColor], NSForegroundColorAttributeName, nil];
		NSMutableAttributedString* numberString = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%lu", (unsigned long)[dragRows count]] attributes:normalAttrs];
		[numberString drawAtPoint:NSMakePoint(47.0f, 16.0f)];
		[image unlockFocus];
		
		return image;
	}
	else
		return [NSImage imageNamed:@"NSMultipleDocuments"];
}


@end


