
#import "GBImageBrowserDataSource.h"
#import "GBGameTableDataSource.h"
#import "GameBaseAppDelegate.h"
#import "CICCS64ColorCorrectionFilter.h"
#import "GBGameCollection.h"

@implementation GBImageBrowserItem

@synthesize path = _path;
@synthesize title = _title;
@synthesize subTitle = _subTitle;


// ----------------------------------------------------------------------------
- (id) initWithPath:(NSString*)thePath andTitle:(NSString*)theTitle andSubTitle:(NSString*)theSubTitle
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		_path = thePath;
		_title = theTitle;
		_subTitle = theSubTitle;
	}
	return self;
}


// IKImageBrowserItem protocol

// ----------------------------------------------------------------------------
- (NSString*) imageRepresentationType
// ----------------------------------------------------------------------------
{
	return IKImageBrowserPathRepresentationType;
}


// ----------------------------------------------------------------------------
- (id) imageRepresentation
// ----------------------------------------------------------------------------
{
	return _path;
}


// ----------------------------------------------------------------------------
- (NSString*) imageUID
// ----------------------------------------------------------------------------
{
	return _path;
}


// ----------------------------------------------------------------------------
- (NSString *) imageTitle
// ----------------------------------------------------------------------------
{
	return _title;
}


// ----------------------------------------------------------------------------
- (NSString *) imageSubtitle
// ----------------------------------------------------------------------------
{
	return _subTitle;
}


// ----------------------------------------------------------------------------
- (NSString *) description
// ----------------------------------------------------------------------------
{
    return [NSString stringWithFormat:@"%@ (%@)\n", _title, _path];
}


@end

#pragma mark -

@implementation GBImageBrowserDataSource


@synthesize browserData = _browserData;
@synthesize unfilteredBrowserData = _unfilteredBrowserData;
@synthesize unfilteredSearchBrowserData = _unfilteredSearchBrowserData;
@synthesize browserView = _browserView;
@synthesize tableDataSource = _tableDataSource;


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	_browserData = [NSMutableArray arrayWithCapacity:20000];
	_unfilteredBrowserData = nil;
	_unfilteredSearchBrowserData = nil;
}


// ----------------------------------------------------------------------------
- (NSUInteger) numberOfItemsInImageBrowser:(IKImageBrowserView*)view
// ----------------------------------------------------------------------------
{
    return [_browserData count];
}


// ----------------------------------------------------------------------------
- (id) imageBrowser:(IKImageBrowserView*)view itemAtIndex:(NSUInteger)index
// ----------------------------------------------------------------------------
{
    return [_browserData objectAtIndex:index];
}


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)view removeItemsAtIndexes:(NSIndexSet*)indexes
// ----------------------------------------------------------------------------
{
	[_browserData removeObjectsAtIndexes:indexes];
}


// ----------------------------------------------------------------------------
- (void) reloadData
// ----------------------------------------------------------------------------
{
	[_browserView reloadData];
}


// ----------------------------------------------------------------------------
- (IBAction) zoomSliderDidChange:(id)sender
// ----------------------------------------------------------------------------
{
    [_browserView setZoomValue:[sender floatValue]];
    [_browserView setNeedsDisplay:YES];
}



// delegate protocol


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)browser cellWasRightClickedAtIndex:(NSUInteger)index withEvent:(NSEvent*)event;
// ----------------------------------------------------------------------------
{
	NSMenu* menu = [browser menuForEvent:event];
	[NSMenu popUpContextMenu:menu withEvent:event forView:browser];
	
}


// ----------------------------------------------------------------------------
- (void) imageBrowser:(IKImageBrowserView*)browser cellWasDoubleClickedAtIndex:(NSUInteger)index
// ----------------------------------------------------------------------------
{
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	[gamebaseAppDelegate playGame:self];
}


// ----------------------------------------------------------------------------
- (void) imageBrowserSelectionDidChange:(IKImageBrowserView*)browser
// ----------------------------------------------------------------------------
{
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	if (!gamebaseAppDelegate.gameTableLoaded)
	{
		[browser setSelectionIndexes:nil byExtendingSelection:NO];
		return;
	}
	
	NSIndexSet* selectedIndices = [browser selectionIndexes];
	
	NSUInteger selectedIndex = [selectedIndices firstIndex];
	if (selectedIndex != NSNotFound)
	{
		[_tableBrowserView selectRowIndexes:selectedIndices byExtendingSelection:NO];
		[_tableBrowserView scrollRowToVisible:selectedIndex];
	}
	else
		[_tableBrowserView deselectAll:self];
}


// ----------------------------------------------------------------------------
- (NSUInteger) imageBrowser:(IKImageBrowserView*)browser writeItemsAtIndexes:(NSIndexSet*)itemIndexes toPasteboard:(NSPasteboard*)pasteboard;
// ----------------------------------------------------------------------------
{
	NSMutableArray* gameIds = [[NSMutableArray alloc] init];
	NSString* firstItemGamePath = nil;
	
	NSUInteger index = [itemIndexes firstIndex];
	while (index != NSNotFound)
	{
		NSMutableDictionary* gameRow = [_tableDataSource.gameTable objectAtIndex:index];
		[gameIds addObject:[gameRow objectForKey:@"GA_Id"]];
		
		if (firstItemGamePath == nil)
			firstItemGamePath = [gameRow objectForKey:@"FullGamePath"];
		
		index = [itemIndexes indexGreaterThanIndex:index];
	}
	
	_tableDataSource.draggedItems = gameIds;
	
    // Provide data for our custom type, and simple NSStrings.
    [pasteboard declareTypes:[NSArray arrayWithObjects:GBGamePBoardType, NSStringPboardType, nil] owner:self];
    [pasteboard setData:[NSData data] forType:GBGamePBoardType];
	
	if (firstItemGamePath != nil)
		[pasteboard setString:firstItemGamePath forType:NSStringPboardType];
	
    return [gameIds count];
}


@end


#pragma mark -

@implementation GBImageBrowserCell


/*
// ----------------------------------------------------------------------------
- (CALayer*) layerForType:(NSString*)type
// ----------------------------------------------------------------------------
{
	NSRect frame = [self frame];
	NSRect imageFrame = [self imageFrame];
	//NSRect relativeImageFrame = NSMakeRect(imageFrame.origin.x - frame.origin.x, imageFrame.origin.y - frame.origin.y, imageFrame.size.width, imageFrame.size.height);
	
	if ([type isEqualToString:IKImageBrowserCellForegroundLayer])
	{
		//no foreground layer on place holders
		if([self cellState] != IKImageStateReady)
			return nil;
		
		//create a foreground layer that will contain several childs layer
		CALayer* layer = [CALayer layer];
		layer.frame = CGRectMake(0, 0, frame.size.width, frame.size.height);
		
		NSRect imageContainerFrame = [self imageContainerFrame];
		NSRect relativeImageContainerFrame = NSMakeRect(imageContainerFrame.origin.x - frame.origin.x, imageContainerFrame.origin.y - frame.origin.y, imageContainerFrame.size.width, imageContainerFrame.size.height);
		
		[CICCS64ColorCorrectionFilter class];
		CIFilter* filter = [CIFilter filterWithName:@"CIColorInvert"]; 
		[filter setDefaults]; 
		[filter setName:@"colorCorrection"]; 
		layer.filters = [NSArray arrayWithObject:filter];

		return layer;
	}
	
	return [super layerForType:type];
}
*/
 
@end


#pragma mark -

@implementation GBImageBrowserView


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[self setValue:[NSColor colorWithCalibratedWhite:0.9f alpha:1.0f] forKey:IKImageBrowserBackgroundColorKey];
	[self setValue:[NSColor colorWithCalibratedWhite:0.2f alpha:1.0f] forKey:IKImageBrowserSelectionColorKey];
	[self setValue:[NSColor colorWithCalibratedWhite:0.0f alpha:1.0f] forKey:IKImageBrowserCellsOutlineColorKey];

	NSMutableDictionary* titleAttributes = [[self valueForKey:IKImageBrowserCellsTitleAttributesKey] mutableCopy];
	[titleAttributes setObject:[NSColor colorWithCalibratedWhite:0.0f alpha:1.0f] forKey:NSForegroundColorAttributeName];
	[self setValue:titleAttributes forKey:IKImageBrowserCellsTitleAttributesKey];

	NSMutableDictionary* highlightedTitleAttributes = [[self valueForKey:IKImageBrowserCellsHighlightedTitleAttributesKey] mutableCopy];
	[highlightedTitleAttributes setObject:[NSColor colorWithCalibratedWhite:1.0f alpha:1.0f] forKey:NSForegroundColorAttributeName];
	[self setValue:highlightedTitleAttributes forKey:IKImageBrowserCellsHighlightedTitleAttributesKey];

	NSMutableDictionary* subtitleAttributes = [[self valueForKey:IKImageBrowserCellsSubtitleAttributesKey] mutableCopy];
	[subtitleAttributes setObject:[NSColor colorWithCalibratedWhite:0.2f alpha:1.0f] forKey:NSForegroundColorAttributeName];
	[self setValue:subtitleAttributes forKey:IKImageBrowserCellsSubtitleAttributesKey];
}	


// ----------------------------------------------------------------------------
- (IKImageBrowserCell *) newCellForRepresentedItem:(id) cell
// ----------------------------------------------------------------------------
{
	return [[GBImageBrowserCell alloc] init];
}


// ----------------------------------------------------------------------------
- (NSMenu*) menuForEvent:(NSEvent*)event
// ----------------------------------------------------------------------------
{
	NSPoint position = [self convertPoint:[event locationInWindow] fromView:nil];
	NSInteger row = [self indexOfItemAtPoint:position];

    BOOL isCollection = [((GBImageBrowserDataSource*)self.dataSource).tableDataSource currentCollection] != nil;
    BOOL isSmartCollection = [[((GBImageBrowserDataSource*)self.dataSource).tableDataSource currentCollection] isSmart];
    BOOL hideMenuItem = (!isCollection) || isSmartCollection;

    NSMenuItem* menuItem = [contextMenu itemWithTag:1100];
    if (menuItem != nil)
        [menuItem setHidden:hideMenuItem];
    
    menuItem = [contextMenu itemWithTag:1101];
    if (menuItem != nil)
        [menuItem setHidden:hideMenuItem];
    
	if (row != NSNotFound)
	{
		NSIndexSet* set = [self selectionIndexes];
		if (![set containsIndex:row])
			[self setSelectionIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
        
		return contextMenu;
	}
		
	return [super menuForEvent:event];
}


// ----------------------------------------------------------------------------
- (void) keyDown:(NSEvent*)event
// ----------------------------------------------------------------------------
{
	NSString* characters = [event charactersIgnoringModifiers];
	NSUInteger modifiers = [event modifierFlags];
	unichar character = [characters characterAtIndex:0];
	
	// Alt-Cursor keys have no effect (used by menu command)
	if ((character == 0xf702 || character == 0xf703) && (modifiers&NSAlternateKeyMask))
		return;
	
	if (character == '\r' || character == 3)
	{
		NSIndexSet* set = [self selectionIndexes];
		if ([set count] == 1)
		{
			GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
			[gamebaseAppDelegate playGame:self];
			return;
		}
	}
	else if (character == 0xf728 || character == 0x007f)
	{
		GBImageBrowserDataSource* dataSource = (GBImageBrowserDataSource*)[self dataSource];
		[dataSource.tableDataSource deleteSelectedItems:self];
		return;
	}
	
	[super keyDown:event];
}


@end


