#import "GBPreferencesController.h"
#import "GBSourceListDataSource.h"
#import "GBSourceListItem.h"


static NSString* GBDefaultKeyDatabasesKey						= @"Databases";
static NSString* GBSearchTypeKey								= @"SearchType";
static NSString* GBEmulatorToolPathKey							= @"EmulatorToolPath";
static NSString* GBSidPlayerToolPathKey							= @"SidPlayerToolPath";
static NSString* GBEmulatorTypeKey								= @"EmulatorType";
static NSString* GBLastGameIdKey								= @"LastGameIdKey";
static NSString* GBImageGridMode								= @"ImageGridMode";
static NSString* GBLaunchSidPlayerInBackground					= @"LaunchSidPlayerInBackground";
static NSString* GBPalEffect									= @"PalEffect";
static NSString* GBColorCorrection								= @"ColorCorrection";

Preferences	gPreferences;


// ----------------------------------------------------------------------------
void Preferences::initializeDefaults()
// ----------------------------------------------------------------------------
{
	mDatabases = [[NSMutableArray alloc] init];
	mSearchType = SEARCH_NAME;
	mEmulatorToolPath = nil;
	mSidPlayerToolPath = nil;
	mEmulatorType = EMULATOR_UNKNOWN;
	mLastViewedGameId = -1;
	mImageGridMode = NO;
	mLaunchSidPlayerInBackground = NO;
	mPalEffectEnabled = YES;
	mColorCorrectionEnabled = YES;
}



@implementation GBPreferencesController

static GBPreferencesController* sharedInstance = nil;


// ----------------------------------------------------------------------------
+ (void) initialize
// ----------------------------------------------------------------------------
{
	gPreferences.initializeDefaults();

    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSMutableDictionary *appDefaults = [[NSMutableDictionary alloc] init];
    
    [appDefaults setObject:[[NSArray alloc] init] forKey:GBDefaultKeyDatabasesKey];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mSearchType] forKey:GBSearchTypeKey];
	
	if (gPreferences.mEmulatorToolPath != nil)
		[appDefaults setObject:gPreferences.mEmulatorToolPath forKey:GBEmulatorToolPathKey];
	
	if (gPreferences.mSidPlayerToolPath != nil)
		[appDefaults setObject:gPreferences.mSidPlayerToolPath forKey:GBSidPlayerToolPathKey];
	
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mEmulatorType] forKey:GBEmulatorTypeKey];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mLastViewedGameId] forKey:GBLastGameIdKey];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mImageGridMode] forKey:GBImageGridMode];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mLaunchSidPlayerInBackground] forKey:GBLaunchSidPlayerInBackground];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mPalEffectEnabled] forKey:GBPalEffect];
    [appDefaults setObject:[NSNumber numberWithInt:gPreferences.mColorCorrectionEnabled] forKey:GBColorCorrection];
	
    [defaults registerDefaults:appDefaults];
}


// ----------------------------------------------------------------------------
+ (GBPreferencesController*) sharedInstance
// ----------------------------------------------------------------------------
{
	if (sharedInstance == nil)
		sharedInstance = [[GBPreferencesController alloc] init];
		
	return sharedInstance;
}


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		loaded = NO;
	}
	return self;
}


// ----------------------------------------------------------------------------
- (void) load
// ----------------------------------------------------------------------------
{
	if (loaded)
		return;

	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	if (defaults == nil)
		return;
	
	gPreferences.mDatabases = [[defaults arrayForKey:GBDefaultKeyDatabasesKey] mutableCopy];
	gPreferences.mSearchType = (GBSearchType) [defaults integerForKey:GBSearchTypeKey];
	gPreferences.mEmulatorToolPath = [defaults objectForKey:GBEmulatorToolPathKey];
	gPreferences.mSidPlayerToolPath = [defaults objectForKey:GBSidPlayerToolPathKey];
	gPreferences.mEmulatorType = (GBEmulatorType) [defaults integerForKey:GBEmulatorTypeKey];
	gPreferences.mLastViewedGameId = [defaults integerForKey:GBLastGameIdKey];
	gPreferences.mImageGridMode = [defaults integerForKey:GBImageGridMode];
	gPreferences.mLaunchSidPlayerInBackground = [defaults integerForKey:GBLaunchSidPlayerInBackground];
	gPreferences.mPalEffectEnabled = [defaults integerForKey:GBPalEffect];
	gPreferences.mColorCorrectionEnabled = [defaults integerForKey:GBColorCorrection];
	
	loaded = YES;
}


// ----------------------------------------------------------------------------
- (void) save
// ----------------------------------------------------------------------------
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	if (defaults == nil)
		return;
		
	[defaults setObject:gPreferences.mDatabases forKey:GBDefaultKeyDatabasesKey];
	[defaults setInteger:gPreferences.mSearchType forKey:GBSearchTypeKey];
	
	if (gPreferences.mEmulatorToolPath != nil)
		[defaults setObject:gPreferences.mEmulatorToolPath forKey:GBEmulatorToolPathKey];
	
	if (gPreferences.mSidPlayerToolPath != nil)
		[defaults setObject:gPreferences.mSidPlayerToolPath forKey:GBSidPlayerToolPathKey];
	
	[defaults setInteger:gPreferences.mEmulatorType forKey:GBEmulatorTypeKey];
	[defaults setObject:[NSNumber numberWithInt:gPreferences.mLastViewedGameId] forKey:GBLastGameIdKey];
    [defaults setObject:[NSNumber numberWithInt:gPreferences.mImageGridMode] forKey:GBImageGridMode];
    [defaults setObject:[NSNumber numberWithInt:gPreferences.mLaunchSidPlayerInBackground] forKey:GBLaunchSidPlayerInBackground];
    [defaults setObject:[NSNumber numberWithInt:gPreferences.mPalEffectEnabled] forKey:GBPalEffect];
    [defaults setObject:[NSNumber numberWithInt:gPreferences.mColorCorrectionEnabled] forKey:GBColorCorrection];

	[defaults synchronize];
}

@end
