#import "GBPreferencesWindowController.h"
#import "GBPreferencesController.h"
#import "GameBaseAppDelegate.h"


@implementation GBPreferencesWindowController


static NSString* GB_VICE_X64_BundleIdentifier = @"org.viceteam.x64";
static NSString* GB_VICE_X64SC_BundleIdentifier = @"org.viceteam.x64sc";
static NSString* GB_Power64_BundleIdentifier = @"at.infinite-loop.Power64";
static NSString* GB_Micro64_BundleIdentifier = @"com.bero.emulators.micro64";
static NSString* GB_SIDPLAY_BundleIdentifier = @"org.sidmusic.sidplay";


static NSString* GBDefaultKeyDontShowNoEmulatorsAlert = @"GBDefaultKeyDontShowNoEmulatorsAlert";

static BOOL sAlreadyShownNoEmulatorsAlert = NO;;

@synthesize emulatorTools = _emulatorTools;
@synthesize sidPlayerTools = _sidPlayerTools;


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
	if (self = [super initWithWindowNibName:@"Preferences"])
	{
		_emulatorTools = [NSMutableArray arrayWithCapacity:5];
		_sidPlayerTools = [NSMutableArray arrayWithCapacity:5];
		
		[self loadWindow];
	}
	
	return self;
}


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[[GBPreferencesController sharedInstance] load];
	
	[self findTools];
	
	[launchSidPlayerInBackgroundButton setState:gPreferences.mLaunchSidPlayerInBackground ? NSOnState : NSOffState];
}


// ----------------------------------------------------------------------------
- (void) windowDidLoad
// ----------------------------------------------------------------------------
{
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowWillClose:)
												 name:NSWindowWillCloseNotification
											   object:[self window]];
}


// ----------------------------------------------------------------------------
- (void) windowWillClose:(NSNotification*)aNotification
// ----------------------------------------------------------------------------
{
	[[GBPreferencesController sharedInstance] save];
}	



// ----------------------------------------------------------------------------
- (IBAction) showWindow:(id)sender
// ----------------------------------------------------------------------------
{
	[self findTools];
	
	[super showWindow:sender];
}


// ----------------------------------------------------------------------------
- (void) findTools
// ----------------------------------------------------------------------------
{
	if (gPreferences.mEmulatorToolPath != nil && ![[NSFileManager defaultManager] fileExistsAtPath:gPreferences.mEmulatorToolPath])
		gPreferences.mEmulatorToolPath = nil;

	if (gPreferences.mSidPlayerToolPath != nil && ![[NSFileManager defaultManager] fileExistsAtPath:gPreferences.mSidPlayerToolPath])
		gPreferences.mSidPlayerToolPath = nil;
	
    if (emulatorSearchQuery == nil)
    {
        emulatorSearchQuery = [[NSMetadataQuery alloc] init];
        [emulatorSearchQuery setDelegate:self];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(emulatorSearchQueryNotification:) name:nil object:emulatorSearchQuery];
    }
	NSPredicate* emulatorSearchPredicate = [NSPredicate predicateWithFormat:@"(kMDItemCFBundleIdentifier == %@) || (kMDItemCFBundleIdentifier == %@) || (kMDItemCFBundleIdentifier == %@) || (kMDItemCFBundleIdentifier == %@)",
											GB_VICE_X64_BundleIdentifier, GB_VICE_X64SC_BundleIdentifier, GB_Power64_BundleIdentifier, GB_Micro64_BundleIdentifier];
	[emulatorSearchQuery setPredicate:emulatorSearchPredicate];
	[emulatorSearchQuery startQuery]; 

    if (sidPlayerSearchQuery == nil)
    {
        sidPlayerSearchQuery = [[NSMetadataQuery alloc] init];
        [sidPlayerSearchQuery setDelegate:self];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(sidPlayerSearchQueryNotification:) name:nil object:sidPlayerSearchQuery];
    }
	NSPredicate* sidPlayerSearchPredicate = [NSPredicate predicateWithFormat:@"kMDItemCFBundleIdentifier == %@", GB_SIDPLAY_BundleIdentifier];
	[sidPlayerSearchQuery setPredicate:sidPlayerSearchPredicate];
	[sidPlayerSearchQuery startQuery]; 
}


// ----------------------------------------------------------------------------
- (NSMetadataItem*) newestItemOfMetadataResults:(NSArray*)results
// ----------------------------------------------------------------------------
{
	NSDate* newestDate = [NSDate distantPast]; 
	NSMetadataItem* newestItem = nil;
	for (NSMetadataItem* item in results)
	{
		NSDate* lastUsedDate = [item valueForAttribute:@"kMDItemLastUsedDate"];
		if ([lastUsedDate compare:newestDate] == NSOrderedDescending)
		{
			newestDate = lastUsedDate;
			newestItem = item;
		}
	}
	
	return newestItem;
}

// ----------------------------------------------------------------------------
- (GBEmulatorType) getEmulatorTypeFromBundleIdentifier:(NSString*)bundleIdentifier
// ----------------------------------------------------------------------------
{
	if ([bundleIdentifier caseInsensitiveCompare:GB_VICE_X64_BundleIdentifier] == NSOrderedSame)
		return EMULATOR_VICE_X64;
	else if ([bundleIdentifier caseInsensitiveCompare:GB_VICE_X64SC_BundleIdentifier] == NSOrderedSame)
		return EMULATOR_VICE_X64SC;
    else if ([bundleIdentifier caseInsensitiveCompare:GB_Micro64_BundleIdentifier] == NSOrderedSame)
        return EMULATOR_MICRO64;
	else if ([bundleIdentifier caseInsensitiveCompare:GB_Power64_BundleIdentifier] == NSOrderedSame)
		return EMULATOR_POWER64;
	
	return EMULATOR_UNKNOWN;
}


// ----------------------------------------------------------------------------
- (NSString*) getEmulatorNameFromType:(GBEmulatorType)type
// ----------------------------------------------------------------------------
{
	switch (type)
	{
		case EMULATOR_UNKNOWN:
			return nil;
			
		case EMULATOR_VICE_X64:
			return @"VICE (x64)";
			break;

		case EMULATOR_VICE_X64SC:
			return @"VICE (x64sc)";
			break;
			
		case EMULATOR_POWER64:
			return @"Power64";
			break;
			
        case EMULATOR_MICRO64:
            return @"Micro64";
            
		default:
			return nil;
			break;
	}
}



// ----------------------------------------------------------------------------
- (NSString*) getNameOfBundle:(NSBundle*)bundle
// ----------------------------------------------------------------------------
{
	NSString*name = [bundle objectForInfoDictionaryKey:@"CFBundleDisplayName"];
	if (name) 
		return name;
	
	name = [bundle objectForInfoDictionaryKey:@"CFBundleName"];
	if (name)
		return name;
	
	return [[[NSFileManager defaultManager] displayNameAtPath:[bundle bundlePath]] stringByDeletingPathExtension];
}


// ----------------------------------------------------------------------------
- (NSString*) getVersionStringOfBundle:(NSBundle*)bundle
// ----------------------------------------------------------------------------
{
	NSString* version = [bundle objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
	if (version != nil)
		return version;
	
	version = [bundle objectForInfoDictionaryKey:@"CFBundleVersion"];
	if (version != nil)
		return version;
	
	return @"Unknown version";
}


// ----------------------------------------------------------------------------
- (void) fillPopupMenuFromMetadataResults:(NSArray*)results isEmulator:(BOOL)isEmulator
// ----------------------------------------------------------------------------
{
	NSMetadataItem* newestItem = [self newestItemOfMetadataResults:results];
	
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
	
	NSInteger selectedIndex = -1;
	NSInteger latestUsedIndex = 0;
	NSInteger index = 0;

	for (NSMetadataItem* item in results)
	{
		NSString* path = [item valueForAttribute:@"kMDItemPath"];
		NSString* bundleIdentifier = [item valueForAttribute:@"kMDItemCFBundleIdentifier"];
		NSBundle* bundle = [NSBundle bundleWithPath:path];
		NSString* versionString = [self getVersionStringOfBundle:bundle];
		BOOL latestOneUsed = (item == newestItem);
		NSString* userName = nil;
		
		if (isEmulator)
		{
			GBEmulatorType type = [self getEmulatorTypeFromBundleIdentifier:bundleIdentifier];
			userName = [self getEmulatorNameFromType:type];
		}
		else 
			userName = [self getNameOfBundle:bundle];

		NSString* descriptiveName = [NSString stringWithFormat:@"%@ %@ %@", userName, versionString, latestOneUsed ? @"(latest one used)" : @""];
		
        //NSLog(@"Found tool: %@ %@ (%@) %@\n", isEmulator ? @"Emulator" : @"SidPlayer", descriptiveName, bundleIdentifier, path);

		GBTool* tool = nil;
		if (isEmulator)
		{
			GBEmulatorType type = [self getEmulatorTypeFromBundleIdentifier:bundleIdentifier];
			tool = [[GBEmulatorTool alloc] initWithName:descriptiveName andPath:path andType:type];
		}
		else
			tool = [[GBSidPlayerTool alloc] initWithName:descriptiveName andPath:path];

		tool.latestOneUsed = latestOneUsed;

		NSMenuItem* menuItem = nil;
		
		if (isEmulator)
		{
			[_emulatorTools addObject:tool];
			menuItem = [[NSMenuItem alloc] initWithTitle:descriptiveName action:nil keyEquivalent:@""];
		}
		else
		{
			[_sidPlayerTools addObject:tool];
			menuItem = [[NSMenuItem alloc] initWithTitle:descriptiveName action:nil keyEquivalent:@""];
		}
		
		[menuItem setTarget:self];
		[menu addItem:menuItem];
		
		NSString* toolPath = isEmulator ? gPreferences.mEmulatorToolPath : gPreferences.mSidPlayerToolPath;
		if (toolPath != nil && [toolPath caseInsensitiveCompare:path] == NSOrderedSame)
			selectedIndex = index;
		
		if (latestOneUsed)
			latestUsedIndex = index;
		
		index++;
	}
	
	NSInteger chosenIndex = selectedIndex != -1 ? selectedIndex : latestUsedIndex;
	
	if (isEmulator)
	{
		if (gPreferences.mEmulatorToolPath == nil)
			gPreferences.mEmulatorToolPath = [[_emulatorTools objectAtIndex:chosenIndex] path];

		[emulatorPopUpButton setMenu:menu];
		[emulatorPopUpButton selectItemAtIndex:chosenIndex];
		[emulatorPopUpButton setEnabled:([[menu itemArray] count] > 0)];
	}
	else 
	{
		if (gPreferences.mSidPlayerToolPath == nil)
			gPreferences.mSidPlayerToolPath = [[_sidPlayerTools objectAtIndex:chosenIndex] path];

		[sidPlayerPopUpButton setMenu:menu];
		[sidPlayerPopUpButton selectItemAtIndex:chosenIndex];
		[sidPlayerPopUpButton setEnabled:([[menu itemArray] count] > 0)];
	}
	
	[self updateToolsIcons];
}


// ----------------------------------------------------------------------------
- (void) emulatorSearchQueryNotification:(NSNotification *)notification
// ----------------------------------------------------------------------------
{
	NSMetadataQuery* query = [notification object];
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	
	if ([[notification name] isEqualToString:NSMetadataQueryDidFinishGatheringNotification])
	{
		NSArray* results = [query results];
		if ([results count] > 0)
		{
			[self fillPopupMenuFromMetadataResults:results isEmulator:YES];
			[emulatorPopUpButton setEnabled:YES];
		}
		else 
		{
			NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
			[menu addItem:[[NSMenuItem alloc] initWithTitle:@"No emulators found" action:nil keyEquivalent:@""]];
			[emulatorPopUpButton setMenu:menu];
			[emulatorPopUpButton setEnabled:NO];
			
			BOOL suppressAlert = [[NSUserDefaults standardUserDefaults] boolForKey:GBDefaultKeyDontShowNoEmulatorsAlert];
			
			if (!suppressAlert && !sAlreadyShownNoEmulatorsAlert)
			{
                NSAlert* alert = [[NSAlert alloc] init];
                [alert addButtonWithTitle:@"OK"];
                [alert addButtonWithTitle:@"Go to VICE webpage"];
                [alert setMessageText:@"No C64 emulators found"];
                [alert setInformativeText:@"GameBase64 Browser was unable to find one of the supported C64 emulators on your Mac!"];
                [alert setAlertStyle:NSWarningAlertStyle];
				[alert setShowsSuppressionButton:YES];
				
				NSInteger returnValue = [alert runModal];

				if ([[alert suppressionButton] state] == NSOnState)
					[[NSUserDefaults standardUserDefaults] setBool:YES forKey:GBDefaultKeyDontShowNoEmulatorsAlert];

				if (returnValue == NSAlertSecondButtonReturn)
					[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://vice-emu.sourceforge.net/macosx.html"]];
				
				sAlreadyShownNoEmulatorsAlert = YES;
			}
		}

		if (![gamebaseAppDelegate.window isVisible])
			[gamebaseAppDelegate.window orderFront:self];
	}
}



// ----------------------------------------------------------------------------
- (void) sidPlayerSearchQueryNotification:(NSNotification *)notification
// ----------------------------------------------------------------------------
{
	NSMetadataQuery* query = [notification object];
	
	if ([[notification name] isEqualToString:NSMetadataQueryDidFinishGatheringNotification])
	{
		NSArray* results = [query results];

		if ([results count] > 0)
			[self fillPopupMenuFromMetadataResults:results isEmulator:NO];
		else
		{
			NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
			[menu addItem:[[NSMenuItem alloc] initWithTitle:@"No music players found" action:nil keyEquivalent:@""]];
			[sidPlayerPopUpButton setMenu:menu];
			[sidPlayerPopUpButton setEnabled:NO];
		}
    }
}


// ----------------------------------------------------------------------------
- (void) updateToolsIcons
// ----------------------------------------------------------------------------
{
	if (gPreferences.mEmulatorToolPath != nil)
	{
		NSImage* icon = [[NSWorkspace sharedWorkspace] iconForFile:gPreferences.mEmulatorToolPath];
		[icon setSize:NSMakeSize(96.0f, 96.0f)];
		[emulatorDropBox setImage:icon];
	}
	
	if (gPreferences.mSidPlayerToolPath != nil)
	{
		NSImage* icon = [[NSWorkspace sharedWorkspace] iconForFile:gPreferences.mSidPlayerToolPath];
		[icon setSize:NSMakeSize(96.0f, 96.0f)];
		[sidPlayerDropBox setImage:icon];
	}
}



// ----------------------------------------------------------------------------
- (IBAction) emulatorToolChanged:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger selectedIndex = [sender indexOfSelectedItem];
	GBEmulatorTool* tool = [_emulatorTools objectAtIndex:selectedIndex];
	
	gPreferences.mEmulatorToolPath = tool.path;
	NSBundle* bundle = [NSBundle bundleWithPath:tool.path];
	gPreferences.mEmulatorType = [self getEmulatorTypeFromBundleIdentifier:[bundle bundleIdentifier]];

	[self updateToolsIcons];
}


// ----------------------------------------------------------------------------
- (IBAction) sidPlayerToolChanged:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger selectedIndex = [sender indexOfSelectedItem];
	GBSidPlayerTool* tool = [_sidPlayerTools objectAtIndex:selectedIndex];
	
	gPreferences.mSidPlayerToolPath = tool.path;

	[self updateToolsIcons];	
}


// ----------------------------------------------------------------------------
- (IBAction) sidPlayerLaunchBehaviourChanged:(id)sender
// ----------------------------------------------------------------------------
{
	gPreferences.mLaunchSidPlayerInBackground = [sender state] == NSOnState;
}


// ----------------------------------------------------------------------------
- (void) toolDroppedOn:(id)sender withPath:(NSString*)path
// ----------------------------------------------------------------------------
{
	if (sender == emulatorDropBox)
	{
		gPreferences.mEmulatorToolPath = path;
		NSBundle* bundle = [NSBundle bundleWithPath:path];
		gPreferences.mEmulatorType = [self getEmulatorTypeFromBundleIdentifier:[bundle bundleIdentifier]];
	}
	else if (sender == sidPlayerDropBox)
	{
		gPreferences.mSidPlayerToolPath = path;
	}
	
	[self updateToolsIcons];
}



@end


@implementation ToolDropBox


// ----------------------------------------------------------------------------
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
// ----------------------------------------------------------------------------
{
	NSString* path = [[[sender draggingPasteboard] propertyListForType:@"NSFilenamesPboardType"] objectAtIndex:0];
	NSString* loweredPath = [[path pathExtension] lowercaseString];
	if ([loweredPath isEqualToString:@"app"])
		return NSDragOperationLink;
	
	return [super draggingEntered:sender];
}


// ----------------------------------------------------------------------------
- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
// ----------------------------------------------------------------------------
{
	NSString* path = [[[sender draggingPasteboard] propertyListForType:@"NSFilenamesPboardType"] objectAtIndex:0];
	if (owner != nil)
		[owner toolDroppedOn:self withPath:path];
}


@end

#pragma mark -

@implementation GBTool

@synthesize name = _name;
@synthesize path = _path;
@synthesize latestOneUsed;


// ----------------------------------------------------------------------------
- (id) initWithName:(NSString*)theName andPath:(NSString*)thePath
// ----------------------------------------------------------------------------
{
	self = [super init];
	if (self != nil)
	{
		_name = theName;
		_path = thePath;
		latestOneUsed = NO;
	}
	return self;
}


@end

#pragma mark -

@implementation GBEmulatorTool

@synthesize type;

// ----------------------------------------------------------------------------
- (id) initWithName:(NSString*)theName andPath:(NSString*)thePath andType:(GBEmulatorType)theType;
// ----------------------------------------------------------------------------
{
	self = [super initWithName:theName andPath:thePath];
	if (self != nil)
		type = theType;

	return self;
}


@end


@implementation GBSidPlayerTool

@end


