#import <Cocoa/Cocoa.h>


@class GBSourceListItem;
@class GBSourceListView;
@class GBGameCollection;
@class GBGameTableDataSource;
@class MAAttachedWindow;


extern NSString* GBSourceListDataBaseItemPBoardType;


@interface GBSourceListDataSource : NSObject 
{
	IBOutlet GBSourceListView* sourceListView;
	IBOutlet NSTableColumn* tableColumn;
	IBOutlet GBGameTableDataSource* gameTableDataSource;

	IBOutlet NSView* databaseGuideView;

	NSMutableArray* rootItems;
	GBSourceListItem* databasesContainerItem;
	GBSourceListItem* collectionsContainerItem;
	
	GBSourceListItem* _currentDatabaseItem;
	GBSourceListItem* _currentCollectionItem;
	NSArray* draggedItems;

    GBGameCollection* _classicsCollection;
    
	MAAttachedWindow* databaseGuideWindow;
}

@property (nonatomic) GBSourceListItem* currentDatabaseItem;
@property (nonatomic) GBSourceListItem* currentCollectionItem;


- (void) initSourceListItems;
- (GBSourceListItem*) addHeaderItemWithName:(NSString*)name atIndex:(NSInteger)index;
- (GBSourceListItem*) addDatabaseItemForPath:(NSString*)path atIndex:(NSInteger)index;
- (GBSourceListItem*) addCollectionItemForPath:(NSString*)path toContainerItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index withCollection:(GBGameCollection*)collection;

+ (GBSourceListItem*) addSourceListItemToItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index forPath:(NSString*)path withName:(NSString*)name withImage:(NSImage*)image;
+ (GBSourceListItem*) addSourceListItemToItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index forPath:(NSString*)path withName:(NSString*)name;

- (void) updateDatabaseItem:(GBSourceListItem*)item asExisting:(BOOL)existing andValid:(BOOL)valid;

- (void) openSourceListItem:(GBSourceListItem*)item;
- (BOOL) removeSourceListItem:(GBSourceListItem*)item;

- (IBAction) removeSelectedSourceListItem:(id)sender;
- (IBAction) revealSelectedSourceListItem:(id)sender;

- (GBSourceListItem*) findItemWithPath:(NSString*)path;
- (GBSourceListItem*) findItemWithPath:(NSString*)path inParentItem:(GBSourceListItem*)parentItem;

- (void) createNewCollectionWithName:(NSString*)name andSelectInSourceList:(BOOL)select;
- (void) sortCollections;

- (GBSourceListView*) sourceListView;
- (NSArray*) draggedItems;

- (GBSourceListItem*) databasesContainerItem;
- (void) updateDatabaseListFromItems;
- (void) selectCurrentDatabaseItem;

- (void) showDatabaseGuide:(BOOL)show;

- (IBAction) addNewCollection:(id)sender;

- (void) addGameIdToClassicsCollection:(NSNumber*)gameId;
- (void) removeGameIdFromClassicsCollection:(NSNumber*)gameId;
- (void) clearClassicsCollection;

@end



@interface GBSourceListView : NSOutlineView
{
	BOOL isActive;
	
	IBOutlet NSMenu* collectionContextMenu;
	IBOutlet NSMenu* databaseContextMenu;
}

- (void) awakeFromNib;
- (void) reloadData;
- (void) itemDidCollapse:(NSNotification*)notification;
- (BOOL) isActive;

@end
