#import "GBSourceListDataSource.h"
#import "GBSourceListItem.h"
#import "GBSourceListCell.h"
#import "GBPreferencesController.h"
#import "GBGameCollection.h"
#import "GameBaseAppDelegate.h"
#import "GBGameTableDataSource.h"
#import "MAAttachedWindow.h"


@implementation GBSourceListDataSource

NSString* GBSourceListDataBaseItemPBoardType = @"GBSourceListDataBaseItemPBoardType";

static NSString* GBDefaultKeyDontShowDeleteCollectionAlert = @"GBDefaultKeyDontShowDeleteCollectionAlert";


@synthesize currentDatabaseItem = _currentDatabaseItem;
@synthesize currentCollectionItem = _currentCollectionItem;


// ----------------------------------------------------------------------------
- (id) init
// ----------------------------------------------------------------------------
{
	if (self = [super init])
	{
		rootItems = [[NSMutableArray alloc] init];
		databasesContainerItem = nil;
		_currentDatabaseItem = nil;
		_currentCollectionItem = nil;
		draggedItems = nil;
        _classicsCollection = nil;
		databaseGuideWindow = nil;
	}
	
	return self;
}


// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	[[GBPreferencesController sharedInstance] load];
	
	[sourceListView registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, NSStringPboardType, GBSourceListDataBaseItemPBoardType, GBGamePBoardType, nil]];
	[sourceListView setVerticalMotionCanBeginDrag:YES];

	[rootItems removeAllObjects];
	
	// Add the container headers
	databasesContainerItem = [self addHeaderItemWithName:@"DATABASES" atIndex:0];
	collectionsContainerItem = [self addHeaderItemWithName:@"COLLECTIONS" atIndex:1];

	[self initSourceListItems];
	[sourceListView reloadData];
	
	NSInteger row = [sourceListView rowForItem:_currentDatabaseItem];
	[sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
}


// ----------------------------------------------------------------------------
- (void) initSourceListItems
// ----------------------------------------------------------------------------
{
	for (NSString* path in gPreferences.mDatabases)
		[self addDatabaseItemForPath:path atIndex:-1];

	// Iterate files in ~/Application Support/GameBase64 Browser/
	NSString* collectionsPath = [GameBaseAppDelegate applicationSupportPath];
	
	NSArray* supportFiles = [[NSFileManager defaultManager] contentsOfDirectoryAtPath:collectionsPath error:NULL];
	for (NSString* supportFile in supportFiles)
	{
		if ([supportFile characterAtIndex:0] == '.')
			continue;
		
		if ([[supportFile pathExtension] caseInsensitiveCompare:[GBGameCollection fileExtension]] != NSOrderedSame)
			continue;
		
		NSString* collectionPath = [collectionsPath stringByAppendingPathComponent:supportFile];;

		GBGameCollection* collection = [GBGameCollection collectionFromFile:collectionPath];
		
		[self addCollectionItemForPath:collectionPath toContainerItem:collectionsContainerItem atIndex:-1 withCollection:collection];
	}
	
    // Add special "Classics" collection
    [self createSmartCollection:@"Classics" andSelectInSourceList:NO];
    
	// check for Favorites collection and create if not found
	NSString* favoritesCollectionName = @"Favorites";
	BOOL foundFavorites = NO;
	NSMutableArray* collectionItems = [collectionsContainerItem children];
	for (GBSourceListItem* item in collectionItems)
	{
		GBGameCollection* collection = (GBGameCollection*) item.representedObject;
		if ([collection.name caseInsensitiveCompare:favoritesCollectionName] == NSOrderedSame)
		{
			foundFavorites = YES;
			break;
		}
	}
	
	if (!foundFavorites)
		[self createNewCollectionWithName:favoritesCollectionName andSelectInSourceList:NO];
	else
		[self sortCollections];
}


// ----------------------------------------------------------------------------
+ (GBSourceListItem*) addSourceListItemToItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index forPath:(NSString*)path withName:(NSString*)name withImage:(NSImage*)image
// ----------------------------------------------------------------------------
{
	BOOL exists = YES;
	
	// If no image is passed, find the icon of the path
	if (image == nil)
	{
		exists = [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:NULL];
		if (exists)
			image = [[NSWorkspace sharedWorkspace] iconForFile:path];
		else
            image = [NSImage imageNamed:@"notfound"];
	}

	NSMutableParagraphStyle* paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
    [paragraphStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];

	NSDictionary* headerAttrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:11.0f], NSFontAttributeName, 
																			paragraphStyle, NSParagraphStyleAttributeName, nil];
	NSAttributedString* listItemName = [[NSAttributedString alloc] initWithString:name attributes:headerAttrs];

	GBSourceListItem* item = [[GBSourceListItem alloc] initWithName:listItemName forPath:path withIcon:image];
	[item setIsPathValid:exists];
	
	if (index == -1)
		[containerItem addChild:item];
	else
		[containerItem insertChild:item atIndex:index];
	
	return item;
}


// ----------------------------------------------------------------------------
+ (GBSourceListItem*) addSourceListItemToItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index forPath:(NSString*)path withName:(NSString*)name
// ----------------------------------------------------------------------------
{
	return [GBSourceListDataSource addSourceListItemToItem:containerItem atIndex:index forPath:path withName:name withImage:nil];
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) addHeaderItemWithName:(NSString*)name atIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	NSMutableParagraphStyle* paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
    [paragraphStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];

	NSDictionary* headerAttrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont boldSystemFontOfSize:11.0f], NSFontAttributeName, 
																			paragraphStyle, NSParagraphStyleAttributeName, nil];
	NSAttributedString* headerName = [[NSAttributedString alloc] initWithString:name attributes:headerAttrs];
	
	GBSourceListItem* headerItem = [[GBSourceListItem alloc] initWithName:headerName forPath:@"" withIcon:nil];
	[headerItem setType:SOURCELIST_HEADER];

	[rootItems insertObject:headerItem atIndex:index];
	
	return headerItem;
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) addDatabaseItemForPath:(NSString*)path atIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	NSString* name = [path lastPathComponent];
	
	BOOL databaseExists = [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:NULL];
	BOOL databaseValid = [GameBaseAppDelegate containsValidDatabaseAtPath:path];

	NSImage* icon = nil;
	if (databaseExists)
        icon = databaseValid ? [NSImage imageNamed:@"database_folder"] : [NSImage imageNamed:@"folder"];
	else
        icon = [NSImage imageNamed:@"notfound"];
	
	GBSourceListItem* item = [GBSourceListDataSource addSourceListItemToItem:databasesContainerItem atIndex:index forPath:path withName:name withImage:icon];
	[item setType:SOURCELIST_DATABASE];
	if ([[databasesContainerItem children] count] == 1)
		_currentDatabaseItem = item;

	return item;	
}


// ----------------------------------------------------------------------------
- (void) updateDatabaseItem:(GBSourceListItem*)item asExisting:(BOOL)existing andValid:(BOOL)valid
// ----------------------------------------------------------------------------
{
	NSImage* icon = nil;
	if (existing)
        icon = valid ? [NSImage imageNamed:@"database_folder"] : [NSImage imageNamed:@"folder"];
	else
        icon = [NSImage imageNamed:@"notfound"];
	
	[item setIcon:icon];
	[sourceListView reloadData];
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) addCollectionItemForPath:(NSString*)path toContainerItem:(GBSourceListItem*)containerItem atIndex:(NSInteger)index withCollection:(GBGameCollection*)collection
// ----------------------------------------------------------------------------
{
	if (collection == nil)
		return nil;

    NSImage* icon = collection.isSmart ? [NSImage imageNamed:@"star"] : [NSImage imageNamed:@"collection"];
	GBSourceListItem* item = [GBSourceListDataSource addSourceListItemToItem:containerItem atIndex:index forPath:path withName:collection.name withImage:icon];
	[item setType:SOURCELIST_COLLECTION];
	item.representedObject = collection;
	
	return item;
}




// ----------------------------------------------------------------------------
- (GBSourceListItem*) findItemWithPath:(NSString*)path
// ----------------------------------------------------------------------------
{
	return [self findItemWithPath:path inParentItem:nil];
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) findItemWithPath:(NSString*)path inParentItem:(GBSourceListItem*)parentItem
// ----------------------------------------------------------------------------
{
	NSArray* items = nil;

	if (parentItem == nil)
		items = rootItems;
	else
		items = [parentItem children];

	for (GBSourceListItem* item in items)
	{
		if ([[item path] caseInsensitiveCompare:path] == NSOrderedSame)
			return item;

		GBSourceListItem* result = [self findItemWithPath:path inParentItem:item];
		if (result != nil)
			return result;
	}
	
	return nil;
}


// ----------------------------------------------------------------------------
- (void) openSourceListItem:(GBSourceListItem*)item
// ----------------------------------------------------------------------------
{
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	
	SourceListItemType type = item.type;
	
	switch (type)
	{
		case SOURCELIST_DATABASE:
		{
			if (item == _currentDatabaseItem)
				[gameTableDataSource showUnfilteredGameTable];
			else
			{
				self.currentDatabaseItem = item;
				[gamebaseAppDelegate loadDatabase:item];
			}
			break;
		}
			
		case SOURCELIST_COLLECTION:
		{
			if (item != _currentDatabaseItem)
			{
				self.currentCollectionItem = item;
				[gamebaseAppDelegate loadCollection:item];
			}
			break;
		}
            
        default:
            break;
	}
}


// ----------------------------------------------------------------------------
- (BOOL) removeSourceListItem:(GBSourceListItem*)item
// ----------------------------------------------------------------------------
{
	if (item == nil)
		return NO;
	
	BOOL itemWasRemoved = NO;
	
	GBSourceListItem* container = nil;
	if ([item isDatabaseItem])
	{
		container = databasesContainerItem;
		[[container children] removeObject:item];
		
		[self updateDatabaseListFromItems];
		
		itemWasRemoved = YES;
	}
	else if ([item isCollectionItem])
	{
		BOOL suppressAlert = [[NSUserDefaults standardUserDefaults] boolForKey:GBDefaultKeyDontShowDeleteCollectionAlert];
		BOOL deleteCollection = YES;
		
		GBGameCollection* collection = (GBGameCollection*) item.representedObject;
		
		if ([collection.gameIds count] == 0)
			suppressAlert = YES;
		
		if (!suppressAlert)
		{
			deleteCollection = NO;
			
			NSAlert* alert = [[NSAlert alloc] init];
            [alert addButtonWithTitle:@"Cancel"];
            [alert addButtonWithTitle:@"Delete"];
            [alert setMessageText:[NSString stringWithFormat:@"Are you sure you want to delete the collection \"%@\" ?", collection.name]];
            [alert setInformativeText:@"If you delete the collection, it will not be possible to undo the operation!"];
            [alert setAlertStyle:NSWarningAlertStyle];
			[alert setShowsSuppressionButton:YES];
			
			if ([alert runModal] == NSAlertSecondButtonReturn)
			{
				if ([[alert suppressionButton] state] == NSOnState)
					[[NSUserDefaults standardUserDefaults] setBool:YES forKey:GBDefaultKeyDontShowDeleteCollectionAlert];
				
				deleteCollection = YES;
			}
		}
		
		if (deleteCollection)
		{
			[[NSFileManager defaultManager] removeItemAtPath:collection.path error:NULL];
			
			[[collectionsContainerItem children] removeObject:item];
			itemWasRemoved = YES;
		}
	}
	
	return itemWasRemoved;
}


// ----------------------------------------------------------------------------
- (void) createNewCollectionWithName:(NSString*)name andSelectInSourceList:(BOOL)select
// ----------------------------------------------------------------------------
{
	GBGameCollection* collection = [[GBGameCollection alloc] init];
    collection.isSmart = NO;
	collection.name = name;
	[collection saveToFile];
	GBSourceListItem* item = [self addCollectionItemForPath:collection.path toContainerItem:collectionsContainerItem atIndex:-1 withCollection:collection];
	
	[self sortCollections];
	[sourceListView reloadData];
    
	if (select)
	{
		NSInteger newRow = [sourceListView rowForItem:item];
		if (newRow >= 0)
		{
			[sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:newRow] byExtendingSelection:NO];
			[sourceListView editColumn:0 row:newRow withEvent:nil select:YES];
		}
	}
}


// ----------------------------------------------------------------------------
- (void) createSmartCollection:(NSString*)name andSelectInSourceList:(BOOL)select
// ----------------------------------------------------------------------------
{
    GBGameCollection* collection = [[GBGameCollection alloc] init];
    collection.isSmart = YES;
    collection.name = name;
    GBSourceListItem* item = [self addCollectionItemForPath:nil toContainerItem:collectionsContainerItem atIndex:-1 withCollection:collection];
    
    _classicsCollection = collection;
    
    [self sortCollections];
    [sourceListView reloadData];
    
    if (select)
    {
        NSInteger newRow = [sourceListView rowForItem:item];
        if (newRow >= 0)
        {
            [sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:newRow] byExtendingSelection:NO];
            [sourceListView editColumn:0 row:newRow withEvent:nil select:YES];
        }
    }
}


// ----------------------------------------------------------------------------
- (void) addGameIdToClassicsCollection:(NSNumber*)gameId
// ----------------------------------------------------------------------------
{
    if (gameId == nil)
        return;
    
    if (_classicsCollection == nil)
        return;
    
    [_classicsCollection.gameIds addObject:gameId];
}


// ----------------------------------------------------------------------------
- (void) removeGameIdFromClassicsCollection:(NSNumber*)gameId
// ----------------------------------------------------------------------------
{
    if (gameId == nil)
        return;
    
    if (_classicsCollection == nil)
        return;
    
    [_classicsCollection.gameIds removeObject:gameId];
}



// ----------------------------------------------------------------------------
- (void) clearClassicsCollection
// ----------------------------------------------------------------------------
{
    if (_classicsCollection == nil)
        return;
    
    [_classicsCollection.gameIds removeAllObjects];
}


// ----------------------------------------------------------------------------
- (void) sortCollections
// ----------------------------------------------------------------------------
{
	NSMutableArray* collectionItems = [collectionsContainerItem children];
	[collectionItems sortUsingSelector:@selector(compare:)];
}



// ----------------------------------------------------------------------------
- (GBSourceListView*) sourceListView
// ----------------------------------------------------------------------------
{
	return sourceListView;
}


// ----------------------------------------------------------------------------
- (NSArray*) draggedItems
// ----------------------------------------------------------------------------
{
	return draggedItems;
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) databasesContainerItem
// ----------------------------------------------------------------------------
{
	return databasesContainerItem;
}


// ----------------------------------------------------------------------------
- (void) showDatabaseGuide:(BOOL)show
// ----------------------------------------------------------------------------
{
	NSWindow* window = [sourceListView window];

	if (show)
	{
		NSRect itemFrame = [sourceListView frameOfCellAtColumn:0 row:0];
		itemFrame = [sourceListView convertRect:itemFrame toView:[window contentView]];
        NSPoint attachmentPoint = NSMakePoint(NSMidX(itemFrame), NSMidY(itemFrame));
		
		databaseGuideWindow = [[MAAttachedWindow alloc] initWithView:databaseGuideView 
												attachedToPoint:attachmentPoint 
													   inWindow:window
														 onSide:MAPositionRight
													 atDistance:10.0f];
		[databaseGuideWindow setBorderColor:[NSColor colorWithCalibratedWhite:1.0f alpha:1.0f]];
		[databaseGuideWindow setBackgroundColor:[NSColor colorWithCalibratedWhite:0.1f alpha:0.75f]];

		[databaseGuideWindow setViewMargin:2.0f];
		[databaseGuideWindow setBorderWidth:2.0f];
		[databaseGuideWindow setCornerRadius:8.0f];
		[databaseGuideWindow setHasArrow:YES];
		[databaseGuideWindow setDrawsRoundCornerBesideArrow:YES];
		[databaseGuideWindow setArrowBaseWidth:20.0f];
		[databaseGuideWindow setArrowHeight:16.0f];

		[window addChildWindow:databaseGuideWindow ordered:NSWindowAbove];
	}
	else
	{
		[window removeChildWindow:databaseGuideWindow];
		[[databaseGuideWindow animator] setAlphaValue:0.0];
		//[databaseGuideWindow orderOut:self];
		databaseGuideWindow = nil;
	}
}


// ----------------------------------------------------------------------------
- (void) updateDatabaseListFromItems
// ----------------------------------------------------------------------------
{
	[gPreferences.mDatabases removeAllObjects];
	for (GBSourceListItem* databaseItem in [databasesContainerItem children])
		[gPreferences.mDatabases addObject:[databaseItem path]];
}


// ----------------------------------------------------------------------------
- (void) selectCurrentDatabaseItem;
// ----------------------------------------------------------------------------
{
    NSInteger row = [sourceListView rowForItem:_currentDatabaseItem];
    [sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
}



#pragma mark -
#pragma mark UI actions


// ----------------------------------------------------------------------------
- (IBAction) removeSelectedSourceListItem:(id)sender
// ----------------------------------------------------------------------------
{
	GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
	if (!gamebaseAppDelegate.gameTableLoaded)
		return;
	
	NSInteger row = [sourceListView selectedRow];
    if (row != -1)
	{
        GBSourceListItem* item = [sourceListView itemAtRow:row];
		BOOL itemWasRemoved = [self removeSourceListItem:item];

		if (itemWasRemoved)
		{
			[sourceListView deselectRow:row];
			int newRow = row - 1;
			GBSourceListItem* newItem = [sourceListView itemAtRow:newRow];
			if (newItem != nil && ![newItem isHeader])
				[sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:newRow] byExtendingSelection:NO];
			else if (newItem != nil && [newItem isHeader] && [[newItem children] count] > 0)
				[sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:newRow+1] byExtendingSelection:NO];

			[sourceListView reloadData];
		}
	}
}


// ----------------------------------------------------------------------------
- (IBAction) revealSelectedSourceListItem:(id)sender
// ----------------------------------------------------------------------------
{
	NSInteger row = [sourceListView selectedRow];
    if (row != -1)
	{
        GBSourceListItem* item = [sourceListView itemAtRow:row];
		[[NSWorkspace sharedWorkspace] selectFile:[item path] inFileViewerRootedAtPath:@""];
	}
}


// ----------------------------------------------------------------------------
- (IBAction) addNewCollection:(id)sender
// ----------------------------------------------------------------------------
{
	if (![[sourceListView window] isVisible])
		return;
	
	[self createNewCollectionWithName:@"untitled collection" andSelectInSourceList:YES];
}


#pragma mark -
#pragma mark data source methods


// ----------------------------------------------------------------------------
- (NSInteger) outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item 
// ----------------------------------------------------------------------------
{
    if (item == nil)
		return [rootItems count];
	else
		return [[item children] count];
}


// ----------------------------------------------------------------------------
- (BOOL) outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item 
// ----------------------------------------------------------------------------
{
	return [item hasChildren];
}


// ----------------------------------------------------------------------------
- (id) outlineView:(NSOutlineView *)outlineView child:(NSInteger)index ofItem:(id)item 
// ----------------------------------------------------------------------------
{
	if (item == nil)
		return [rootItems objectAtIndex:index];
	else
		return [(GBSourceListItem *)item childAtIndex:index];
}


// ----------------------------------------------------------------------------
- (id) outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item 
// ----------------------------------------------------------------------------
{
    return (item == nil) ? @"no item" : [item name];
}


// ----------------------------------------------------------------------------
- (void)outlineView:(NSOutlineView *)outlineView setObjectValue:(id)object forTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
// ----------------------------------------------------------------------------
{
	if (item == nil)
		return;
	
	GBSourceListItem* sourceListItem = (GBSourceListItem*) item;

	NSMutableParagraphStyle* paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
    [paragraphStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];
	NSDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:11.0f], NSFontAttributeName,
																			paragraphStyle, NSParagraphStyleAttributeName, nil];
	NSAttributedString* listItemName = [[NSAttributedString alloc] initWithString:object attributes:attrs];
	
	if ([listItemName length] == 0)
		return;
	
	[sourceListItem setName:listItemName]; 

	GBGameCollection* collection = (GBGameCollection*) sourceListItem.representedObject;
	if (collection != nil)
	{
		collection.name = object;
		[collection saveToFile];
	}

	[self sortCollections];
	[sourceListView reloadData];
	
    NSInteger newRow = [sourceListView rowForItem:item];
	if (newRow >= 0)
		[sourceListView selectRowIndexes:[NSIndexSet indexSetWithIndex:newRow] byExtendingSelection:NO];
}


#pragma mark -
#pragma mark outlineview delegate methods


// ----------------------------------------------------------------------------
- (BOOL) outlineView:(NSOutlineView *)outlineView shouldEditTableColumn:(NSTableColumn *)tableColumn item:(id)item 
// ----------------------------------------------------------------------------
{
	if ([item isCollectionItem])
		return YES;
	
	return NO;
}



// ----------------------------------------------------------------------------
- (void) outlineView:(NSOutlineView *)outlineView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *)tableColumn item:(id)item
// ----------------------------------------------------------------------------
{
	if (item != nil && cell != nil)
	{
		GBSourceListItem* selectedItem = [outlineView itemAtRow:[outlineView selectedRow]];
		if(selectedItem == item || _currentDatabaseItem == item)
		{
			NSMutableDictionary* attrs = [[[(GBSourceListItem*)item name] attributesAtIndex:0 effectiveRange:NULL] mutableCopy];
			[attrs setObject:[NSFont boldSystemFontOfSize:12.0f] forKey:NSFontAttributeName];
            [attrs setObject:[NSColor colorWithWhite:0.001f alpha:1.0f] forKey:NSForegroundColorAttributeName];
			NSAttributedString* name = [[NSAttributedString alloc] initWithString:[[(GBSourceListItem*)item name] string]  attributes:attrs];
			[cell setAttributedStringValue:name];
		}
		else
        {
            NSMutableDictionary* attrs = [[[(GBSourceListItem*)item name] attributesAtIndex:0 effectiveRange:NULL] mutableCopy];
            [attrs setObject:[NSFont boldSystemFontOfSize:12.0f] forKey:NSFontAttributeName];
            [attrs setObject:[NSColor colorWithWhite:0.001f alpha:1.0f] forKey:NSForegroundColorAttributeName];
			[cell setAttributedStringValue:[(GBSourceListItem*)item name]];
        }

		[cell setImage:[item icon]];
		[cell setLineBreakMode:NSLineBreakByTruncatingTail];
		[cell setFocusRingType:NSFocusRingTypeNone];

		if ([item isHeader])
		{
			GBSourceListView* view = (GBSourceListView*) outlineView;
			if ([view isActive])
				[cell setTextColor:[NSColor colorWithDeviceRed:0.376f green:0.431f blue:0.502f alpha:1.0f]];	
			else
				[cell setTextColor:[NSColor colorWithDeviceRed:0.376f green:0.376f blue:0.376f alpha:1.0f]];			
		}
//        else
//            [cell setTextColor:[NSColor colorWithWhite:0.001f alpha:1.0f]];
	}
}


// ----------------------------------------------------------------------------
- (BOOL) outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item
// ----------------------------------------------------------------------------
{
	if ([item isCollectionItem] || [item isDatabaseItem])
	{
		GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
		return gamebaseAppDelegate.gameTableLoaded;
	}
	
	return (![item isHeader]);
}


// ----------------------------------------------------------------------------
- (BOOL)outlineView:(NSOutlineView *)outlineView writeItems:(NSArray*)items toPasteboard:(NSPasteboard*)pasteboard
// ----------------------------------------------------------------------------
{
	GBSourceListItem* firstItem = (GBSourceListItem*) [items objectAtIndex:0];
	if (![firstItem isDatabaseItem])
		return NO;
	
	draggedItems = items;
    [pasteboard declareTypes:[NSArray arrayWithObjects:GBSourceListDataBaseItemPBoardType, NSStringPboardType, nil] owner:self];
    [pasteboard setData:[NSData data] forType:GBSourceListDataBaseItemPBoardType];
	
	if ([items count] == 1)
		[pasteboard setString:[[items objectAtIndex:0] path] forType:NSStringPboardType];
	
    return YES;
}


// ----------------------------------------------------------------------------
- (NSDragOperation)outlineView:(NSOutlineView *)outlineView validateDrop:(id <NSDraggingInfo>)info proposedItem:(id)item proposedChildIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	GBSourceListItem* proposedItem = item;

	NSString* type = [[info draggingPasteboard] availableTypeFromArray:[NSArray arrayWithObjects:NSFilenamesPboardType, GBSourceListDataBaseItemPBoardType, GBGamePBoardType, nil]];
	if (type == nil)
		return NSDragOperationNone;
	
	if (proposedItem == nil)
		return NSDragOperationNone;
	
	if ([info draggingSource] == outlineView)
	{
		// Only database items can be dragged, and then only within the database section
		if ([type isEqualToString:GBSourceListDataBaseItemPBoardType] && [proposedItem isHeader] && proposedItem == databasesContainerItem && [[draggedItems objectAtIndex:0] isDatabaseItem])
		{
			return NSDragOperationGeneric;
		}
	
		return NSDragOperationNone;
	}
	else
	{
		if ([type isEqualToString:GBGamePBoardType] && [proposedItem isCollectionItem] && index == NSOutlineViewDropOnItemIndex)
		{
            GBGameCollection* collection = (GBGameCollection*) proposedItem.representedObject;
            
            if (collection.isSmart)
                return NSDragOperationNone;
            
			return NSDragOperationCopy;
		}
		else if (([type isEqualToString:NSFilenamesPboardType] || [type isEqualToString:GBSourceListDataBaseItemPBoardType]) && [proposedItem isHeader] && proposedItem == databasesContainerItem)
		{
			// If there are files in the list of paths, we can't accept the drop
			NSPasteboard* pasteBoard = [info draggingPasteboard];
			NSArray* paths = [pasteBoard propertyListForType:NSFilenamesPboardType];
			BOOL isFolder = NO;
			for (NSString* path in paths) 
			{
				[[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&isFolder];
				if (!isFolder)
					break;
				
				for (NSString* databasePath in gPreferences.mDatabases)
				{
					if ([path isEqualToString:databasePath])
						return NSDragOperationNone;
				}
			}
			
			return isFolder ? NSDragOperationGeneric : NSDragOperationNone;
		}
	}
	
	return NSDragOperationNone; 
}


// ----------------------------------------------------------------------------
- (BOOL)outlineView:(NSOutlineView *)outlineView acceptDrop:(id <NSDraggingInfo>)info item:(id)item childIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	NSPasteboard* pasteboard = [info draggingPasteboard];
	NSArray* supportedTypes = [NSArray arrayWithObjects:GBSourceListDataBaseItemPBoardType, NSFilenamesPboardType, GBGamePBoardType, nil];
	NSString* bestType = [pasteboard availableTypeFromArray:supportedTypes];
	if (bestType == nil)
		return NO;
	
	GBSourceListItem* targetItem = item;
	int targetIndex = (index == NSOutlineViewDropOnItemIndex) ? -1 : index;
	NSMutableArray* targetContainer = [targetItem children];
	
	if ([bestType isEqualToString:GBSourceListDataBaseItemPBoardType])
	{
		GBSourceListItem* selectedItem = [outlineView itemAtRow:[outlineView selectedRow]];
		GBSourceListItem* draggedItem = [draggedItems objectAtIndex:0];
		
		NSInteger currentIndex = [targetContainer indexOfObject:draggedItem];
		
		// Remove the dragged item from the array
		[targetContainer removeObject:draggedItem];
		if (targetIndex > currentIndex)
			targetIndex--;

		// Add the dragged item at the desired index
		if (targetIndex >= [targetContainer count])
			[targetContainer addObject:draggedItem];
		else
			[targetContainer insertObject:draggedItem atIndex:targetIndex];
		
		// Rebuild databases preference from datasource
		[self updateDatabaseListFromItems];
		
		[sourceListView reloadData];

		NSInteger row = [outlineView rowForItem:selectedItem];
		[outlineView selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
		
		[[GBPreferencesController sharedInstance] save];
	}
	else if ([bestType isEqualToString:NSFilenamesPboardType])
	{
		NSArray* droppedPaths = [pasteboard propertyListForType:NSFilenamesPboardType];
	
		// Add the paths that are directories to the databases list
		for (NSString* path in droppedPaths)
		{
			BOOL folder = NO;
			BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath:path isDirectory:&folder];
			if (!exists || !folder)
				continue;
		
			GBSourceListItem* newDatabaseItem = [self addDatabaseItemForPath:path atIndex:targetIndex];
			[gPreferences.mDatabases addObject:path];
			
			if ([gPreferences.mDatabases count] == 1)
			{
				[self showDatabaseGuide:NO];
				_currentDatabaseItem = newDatabaseItem;
				GameBaseAppDelegate* gamebaseAppDelegate = (GameBaseAppDelegate*)[NSApp delegate];
				[gamebaseAppDelegate loadDatabase:_currentDatabaseItem];
			}
		}
	}
	else if ([bestType isEqualToString:GBGamePBoardType] && [targetItem isCollectionItem])
	{
		// Add the dragged game ids to the collection
		GBGameCollection* collection = (GBGameCollection*) targetItem.representedObject;
		NSMutableArray* draggedGameIds = [gameTableDataSource.draggedItems mutableCopy];
		
		for (NSNumber* draggedGameId in draggedGameIds)
		{
			NSInteger draggedGameIdValue = [draggedGameId integerValue];
			BOOL alreadyInCollection = NO;
			for (NSNumber* collectionGameId in collection.gameIds)
			{
				if ([collectionGameId integerValue] == draggedGameIdValue)
				{
					alreadyInCollection = YES;
					break;
				}
			}
			
			if (!alreadyInCollection)
				[collection.gameIds addObject:draggedGameId];
		}
		
		[collection saveToFile];
	}
	
	[sourceListView reloadData];

    return YES;
}


// ----------------------------------------------------------------------------
- (BOOL) outlineView:(NSOutlineView *)sender isGroupItem:(id)item
// ----------------------------------------------------------------------------
{
	return [item isHeader];
}


// ----------------------------------------------------------------------------
- (void) outlineViewSelectionDidChange:(NSNotification*)notification
// ----------------------------------------------------------------------------
{
	NSOutlineView* view = [notification object];
	NSInteger selectedRow = [view selectedRow];
	
	if (selectedRow != -1)
	{
		GBSourceListItem* selectedItem = [view itemAtRow:selectedRow];

		if (selectedItem != nil)
		{
			GBSourceListDataSource* sourceListDataSource = [view dataSource];
			[sourceListDataSource openSourceListItem:selectedItem];
		}
		
		[view reloadData];
	}
}

@end

#pragma mark -

@implementation GBSourceListView


// ----------------------------------------------------------------------------
- (void) viewWillMoveToWindow:(NSWindow *)newWindow
// ----------------------------------------------------------------------------
{
	[[NSNotificationCenter defaultCenter] removeObserver:self 
	                                                name:NSWindowDidResignKeyNotification
												  object:nil];
												  
	[[NSNotificationCenter defaultCenter] addObserver:self
	                                         selector:@selector(windowDidChangeKeyNotification:)
											     name:NSWindowDidResignKeyNotification object:newWindow];
												 
	[[NSNotificationCenter defaultCenter] removeObserver:self
													name:NSWindowDidBecomeKeyNotification
												  object:nil];
												  
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(windowDidChangeKeyNotification:)
												 name:NSWindowDidBecomeKeyNotification
											   object:newWindow];
}

// NSTableView

// ----------------------------------------------------------------------------
- (void) awakeFromNib
// ----------------------------------------------------------------------------
{
	isActive = NO;
	[self setDoubleAction:@selector(doubleClick:)];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
	                                         selector:@selector(itemDidCollapse:)
											     name:NSOutlineViewItemDidCollapseNotification object:nil];
}


// ----------------------------------------------------------------------------
- (void) reloadData
// ----------------------------------------------------------------------------
{
	[super reloadData];
		
	GBSourceListDataSource* source = [self dataSource];
	int rootItemCount = [source outlineView:self numberOfChildrenOfItem:nil];
	for (int i = 0; i < rootItemCount; i++)
	{
		GBSourceListItem* item = [source outlineView:self child:i ofItem:nil]; 
		[self expandItem:item];
	}
}


// NSTableView (Private)

// ----------------------------------------------------------------------------
- (void) windowDidChangeKeyNotification:(NSNotification*)notification
// ----------------------------------------------------------------------------
{
	if ([[notification name] isEqualToString:NSWindowDidBecomeKeyNotification])
		isActive = YES;
	else
		isActive = NO;
}


// ----------------------------------------------------------------------------
- (void) keyDown:(NSEvent*)event
// ----------------------------------------------------------------------------
{
	NSString* characters = [event charactersIgnoringModifiers];
	unichar character = [characters characterAtIndex:0];
	
	if (character == 63272 || character == 127)
	{
		[(GBSourceListDataSource*)[self dataSource] removeSelectedSourceListItem:self];
		return;
	}
	
	[super keyDown:event];
}


// ----------------------------------------------------------------------------
- (void) itemDidCollapse:(NSNotification*)notification
// ----------------------------------------------------------------------------
{

}


// ----------------------------------------------------------------------------
- (BOOL) isActive
// ----------------------------------------------------------------------------
{
	return isActive;
}


// ----------------------------------------------------------------------------
- (void)drawRow:(NSInteger)rowIndex clipRect:(NSRect)clipRect
// ----------------------------------------------------------------------------
{
    NSColor* bgColor = Nil;
    
    if (self == [[self window] firstResponder] && [[self window] isMainWindow] && [[self window] isKeyWindow])
        bgColor = [NSColor colorWithCalibratedWhite:0.8f alpha:1.0f];
    else
        bgColor = [NSColor colorWithCalibratedWhite:0.9f alpha:1.0f];
    
    NSIndexSet* selectedRowIndexes = [self selectedRowIndexes];
    if ([selectedRowIndexes containsIndex:rowIndex])
    {
        [bgColor setFill];
        NSRectFill([self rectOfRow:rowIndex]);
    }
    
	[super drawRow:rowIndex clipRect:clipRect];
}




// ----------------------------------------------------------------------------
- (NSMenu*) menuForEvent:(NSEvent *)event
// ----------------------------------------------------------------------------
{
	NSPoint position = [self convertPoint:[event locationInWindow] fromView:nil];
	NSInteger row = [self rowAtPoint:position];
	[self selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];

	GBSourceListItem* item = [self itemAtRow:row];
	
	if (item == nil)
		return nil;

	if ([item isCollectionItem])
    {
        GBGameCollection* collection = (GBGameCollection*) item.representedObject;
        if (collection.isSmart)
            return nil;
        else
            return collectionContextMenu;
    }

	if ([item isDatabaseItem])
		return databaseContextMenu;

	return nil;
}

@end


