#import <Cocoa/Cocoa.h>


enum SourceListItemType
{
	SOURCELIST_INVALID = 0,
	SOURCELIST_HEADER,
	SOURCELIST_DATABASE,
	SOURCELIST_COLLECTION
};


@interface GBSourceListItem : NSObject
{
	SourceListItemType _type;
	NSAttributedString* _name;
	NSString* _path;
	NSImage* _icon;
	NSObject* _representedObject;
	NSMutableArray* _children;
	
	BOOL _isPathValid;
}

@property (nonatomic) NSString* path;
@property (nonatomic) NSImage* icon;
@property (assign) SourceListItemType type;
@property (nonatomic) NSObject* representedObject;
@property (assign) BOOL isPathValid;

- (id) initWithName:(NSAttributedString*)theName forPath:(NSString*)thePath withIcon:(NSImage*)theIcon;
- (void) addChild:(GBSourceListItem*)item;
- (void) insertChild:(GBSourceListItem*)item atIndex:(NSInteger)index;
- (GBSourceListItem*) childAtIndex:(int)index;

- (id) initWithCoder:(NSCoder*)coder;
- (void) encodeWithCoder:(NSCoder*)coder;

- (NSAttributedString*)name;
- (void) setName:(NSAttributedString*)nameString;
- (void) setNameFromString:(NSString*)nameString;

- (BOOL) isHeader;
- (BOOL) isDatabaseItem;
- (BOOL) isCollectionItem;

- (BOOL) hasChildren;
- (NSMutableArray*) children;
- (void) setChildren:(NSMutableArray*)array;

- (NSComparisonResult) compare:(GBSourceListItem *)otherItem;

@end
