#import "GBSourceListItem.h"
#import "GBSourceListDataSource.h"


@implementation GBSourceListItem

@synthesize path = _path;
@synthesize icon = _icon;
@synthesize type = _type;
@synthesize representedObject = _representedObject;
@synthesize isPathValid = _isPathValid;


// ----------------------------------------------------------------------------
- (id) initWithName:(NSAttributedString*)theName forPath:(NSString*)thePath withIcon:(NSImage*)theIcon
// ----------------------------------------------------------------------------
{
    if (self = [super init]) 
	{
		_type = SOURCELIST_INVALID;
        _name = theName;
		_path = thePath;
		_icon = theIcon;
		_isPathValid = NO;
		_children = nil;
    }
    return self;
}


// ----------------------------------------------------------------------------
- (id) initWithCoder:(NSCoder*)coder
// ----------------------------------------------------------------------------
{
	if (self = [super init])
	{
		[self setType:(SourceListItemType)[coder decodeIntForKey:@"type"]];
		[self setName:[coder decodeObjectForKey:@"name"]];
		[self setPath:[coder decodeObjectForKey:@"path"]];
		[self setIcon:[coder decodeObjectForKey:@"icon"]];
		[self setChildren:[coder decodeObjectForKey:@"children"]];
	}
	return self;
}


// ----------------------------------------------------------------------------
- (void) encodeWithCoder:(NSCoder*)coder
// ----------------------------------------------------------------------------
{
	//[super encodeWithCoder:coder];
	
	[coder encodeInt:_type forKey:@"type"];
	[coder encodeObject:_name forKey:@"name"];
	[coder encodeObject:_path forKey:@"path"];
	[coder encodeObject:_icon forKey:@"icon"];
	[coder encodeObject:_children forKey:@"children"];
}


// ----------------------------------------------------------------------------
- (NSAttributedString*) name
// ----------------------------------------------------------------------------
{
	return _name;
}


// ----------------------------------------------------------------------------
- (void) setName:(NSAttributedString*)nameString
// ----------------------------------------------------------------------------
{
	_name = nameString;
}


// ----------------------------------------------------------------------------
- (void) setNameFromString:(NSString*)nameString
// ----------------------------------------------------------------------------
{
	NSMutableParagraphStyle* paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
    [paragraphStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];

	NSDictionary* headerAttrs = [NSDictionary dictionaryWithObjectsAndKeys:[NSFont systemFontOfSize:11.0f], NSFontAttributeName, 
																			paragraphStyle, NSParagraphStyleAttributeName, nil];
	_name = [[NSAttributedString alloc] initWithString:nameString attributes:headerAttrs];
}


// ----------------------------------------------------------------------------
- (BOOL) isHeader
// ----------------------------------------------------------------------------
{
	return _type == SOURCELIST_HEADER;
}


// ----------------------------------------------------------------------------
- (BOOL) isDatabaseItem
// ----------------------------------------------------------------------------
{
	return _type == SOURCELIST_DATABASE;
}


// ----------------------------------------------------------------------------
- (BOOL) isCollectionItem
// ----------------------------------------------------------------------------
{
	return _type == SOURCELIST_COLLECTION;
}


// ----------------------------------------------------------------------------
- (void) addChild:(GBSourceListItem*)item
// ----------------------------------------------------------------------------
{
	if (_children == nil)
		_children = [[NSMutableArray alloc] init];
		
	[_children addObject:item];
}


// ----------------------------------------------------------------------------
- (void) insertChild:(GBSourceListItem*)item atIndex:(NSInteger)index
// ----------------------------------------------------------------------------
{
	if (_children == nil)
		_children = [[NSMutableArray alloc] init];

	[_children insertObject:item atIndex:index];
}


// ----------------------------------------------------------------------------
- (GBSourceListItem*) childAtIndex:(int)index
// ----------------------------------------------------------------------------
{
	if (_children == nil)
		return nil;
	
	if (index < [_children count])
		return [_children objectAtIndex:index];
	else
		return nil;
}


// ----------------------------------------------------------------------------
- (BOOL) hasChildren
// ----------------------------------------------------------------------------
{
	if (_children == nil)
		return NO;

	return ([_children count] > 0);
}


// ----------------------------------------------------------------------------
- (NSMutableArray*) children
// ----------------------------------------------------------------------------
{
	return _children;
}


// ----------------------------------------------------------------------------
- (void) setChildren:(NSMutableArray*)array
// ----------------------------------------------------------------------------
{
	_children = array;
}


// ----------------------------------------------------------------------------
- (NSComparisonResult) compare:(GBSourceListItem *)otherItem;
// ----------------------------------------------------------------------------
{
	return [[_name string] localizedCaseInsensitiveCompare:[[otherItem name] string]];
}

@end
