#import <Cocoa/Cocoa.h>
#import "SQLite3Connector.h"
#import "GBBrowserWindow.h"
#import "GBPreferencesWindowController.h"

@class GBSourceListDataSource;
@class GBGameTableDataSource;
@class GBImageBrowserDataSource;
@class GBGameDetailController;
@class GBGameCollection;
@class GBSourceListItem;


enum ValidationItemTag
{
	VALIDATION_TAG_GAME_INFO = 1001,
	VALIDATION_TAG_VERSION_INFO = 1002,
	VALIDATION_TAG_EXTRAS = 1003,
	VALIDATION_TAG_PLAY_GAME = 1005,
	VALIDATION_TAG_PLAY_MUSIC = 1006,
	VALIDATION_TAG_REVEAL_GAME = 1007,
	VALIDATION_TAG_REVEAL_MUSIC = 1008,
	VALIDATION_TAG_FIND_VIDEO = 1009,
	VALIDATION_TAG_SHOW_WEBPAGE = 1010,
	VALIDATION_TAG_NEXT_SCREENSHOT = 2001,
	VALIDATION_TAG_PREV_SCREENSHOT = 2002
};


@interface GameBaseAppDelegate : NSObject
{
	BOOL _gameTableLoaded;
	NSString* databaseStoragePath;
	NSString* _databasePath;
	SQLite3Connector* _databaseConnector;
	
	GBPreferencesWindowController* prefsWindowController;

	NSTask* unzipTask;
	NSString* unzipDirectoryPath;
	
    GBBrowserWindow* _window;
	NSProgressIndicator* _loadingProgressIndicator;
	IBOutlet NSTextField* loadingLabel;
	
	GBSourceListDataSource* _sourceListDataSource;
	IBOutlet GBGameTableDataSource* gameTableDataSource;
	IBOutlet GBImageBrowserDataSource* imageBrowserDataSource;
	GBGameDetailController* _gameDetailController;

	IBOutlet NSSplitView* sourceListSplitView;
	
	IBOutlet NSView* dataContentView;
	IBOutlet NSView* browserView;
	IBOutlet NSView* imageBrowserView;
	IBOutlet NSView* sourceListView;
	
	IBOutlet NSView* zoomSliderView;
	IBOutlet NSSegmentedControl* viewModeSwitch;
}

@property (assign) BOOL gameTableLoaded;
@property (nonatomic) NSString* databasePath;
@property  SQLite3Connector* databaseConnector;
@property (nonatomic) IBOutlet GBBrowserWindow* window;
@property (nonatomic) IBOutlet GBSourceListDataSource* sourceListDataSource;
@property (nonatomic) IBOutlet NSProgressIndicator* loadingProgressIndicator;
@property (nonatomic) IBOutlet GBGameDetailController* gameDetailController;


+ (BOOL) containsValidDatabaseAtPath:(NSString*)storagePath;
+ (NSString*) databasePathFromDatabaseStoragePath:(NSString*)storagePath;

- (void) loadDatabase:(GBSourceListItem*)databaseItem;
- (void) loadCollection:(GBSourceListItem*)collectionItem;

- (void) loadGameTableThread:(id)object;

- (void) playGame:(id)sender;
- (void) playMusic:(id)sender;
- (void) findYoutubeLongplayForGame:(NSString*)gameName;

- (void) findMissingDatabasePath:(GBSourceListItem*)databaseItem;

- (IBAction) revealGameInFinder:(id)sender;
- (IBAction) revealMusicInFinder:(id)sender;
- (IBAction) copyPathOfGameToClipboard:(id)sender;
- (IBAction) copyPathOfMusicToClipboard:(id)sender;
- (IBAction) copyRelativePathOfMusicToClipboard:(id)sender;

- (IBAction) moveFocusToSearchField:(id)sender;
- (IBAction) viewModeChanged:(id)sender;
- (IBAction) switchToListMode:(id)sender;
- (IBAction) switchToImageGridMode:(id)sender;

- (IBAction) showPreferencesWindow:(id)sender;

+ (NSString*) applicationSupportPath;

@end



@interface GBButtonToolbarItem : NSToolbarItem

- (void) validate;

@end
