
#import "NSString+PathUtilities.h"


#pragma mark -
@implementation NSString (PathUtilities)

// ----------------------------------------------------------------------------
+ (NSString*) commonRootPathOfFilename:(NSString*)filename andFilename:(NSString*)otherFilename
// ----------------------------------------------------------------------------
{
    NSArray* filenameArray = [filename pathComponents];
	NSArray* otherArray = [[otherFilename stringByStandardizingPath] pathComponents];
	
    int minLength = MIN([filenameArray count], [otherArray count]);
	
    NSMutableArray* resultArray = [NSMutableArray arrayWithCapacity:minLength];
	
    for (int i = 0; i < minLength; i++)
        if ([[filenameArray objectAtIndex:i] caseInsensitiveCompare:[otherArray objectAtIndex:i]] == NSOrderedSame)
            [resultArray addObject:[filenameArray objectAtIndex:i]];
	
    if ([resultArray count] == 0)
        return nil;
	
    return [NSString pathWithComponents:resultArray];
}


// ----------------------------------------------------------------------------
- (NSString*) stringByRemovingPrefix:(NSString *)prefix
// ----------------------------------------------------------------------------
{
    NSRange aRange;
	
    aRange = [self rangeOfString:prefix options:NSCaseInsensitiveSearch];
    if ((aRange.length == 0) || (aRange.location != 0))
        return self;
    return [self substringFromIndex:aRange.location + aRange.length];
}


// ----------------------------------------------------------------------------
- (NSString*) stringWithPathRelativeTo:(NSString*)anchorPath
// ----------------------------------------------------------------------------
{
    NSArray *pathComponents = [self pathComponents];
    NSArray *anchorComponents = [anchorPath pathComponents];
    
    NSInteger componentsInCommon = MIN([pathComponents count], [anchorComponents count]);
    for (NSInteger i = 0, n = componentsInCommon; i < n; i++)
    {
        if (![[pathComponents objectAtIndex:i] isEqualToString:[anchorComponents objectAtIndex:i]])
        {
            componentsInCommon = i;
            break;
        }
    }
    
    NSUInteger numberOfParentComponents = [anchorComponents count] - componentsInCommon;
    NSUInteger numberOfPathComponents = [pathComponents count] - componentsInCommon;
    
    NSMutableArray *relativeComponents = [NSMutableArray arrayWithCapacity:
                                          numberOfParentComponents + numberOfPathComponents];

    for (NSInteger i = 0; i < numberOfParentComponents; i++)
        [relativeComponents addObject:@".."];

    [relativeComponents addObjectsFromArray:[pathComponents subarrayWithRange:NSMakeRange(componentsInCommon, numberOfPathComponents)]];
    
    return [NSString pathWithComponents:relativeComponents];
}

@end

