#import "SQLite3Connector.h"


@implementation SQLite3Connector


// --------------------------------------------------------------------------------
+ (NSString*) escapeString:(NSString*)string
// --------------------------------------------------------------------------------
{
	return string;
}


// --------------------------------------------------------------------------------
+ (BOOL) databaseExists:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	BOOL isDir;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	if ([fileManager fileExistsAtPath:dataBasePath isDirectory:&isDir])
		return !isDir;
	else
		return NO;
}


// --------------------------------------------------------------------------------
+ (void) checkDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	if (![self databaseExists:dataBasePath]) NSAssert1(0, @"Database '%@' does not exists", dataBasePath);
}


// --------------------------------------------------------------------------------
+ (NSArray *) getTablesInDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	[self checkDatabase:dataBasePath];
	NSArray* array = [[NSArray alloc] init];
	
	return array;
}


// --------------------------------------------------------------------------------
+ (BOOL) dropDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	[self checkDatabase:dataBasePath];
	NSFileManager *fileManager = [NSFileManager defaultManager];
	return [fileManager removeItemAtPath:dataBasePath error:nil];
	
}


// --------------------------------------------------------------------------------
+ (NSArray *) getDatabaseFilePaths:(NSString*)path
// --------------------------------------------------------------------------------
{
	NSArray* array = [[NSArray alloc] init];
	
	return array;
}


// --------------------------------------------------------------------------------
+ (int) getDatabaseFileSize:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSFileManager *manager = [NSFileManager defaultManager];
	NSDictionary* fileAttributes = [manager attributesOfItemAtPath:dataBasePath error:NULL];
	
	int ret = [[fileAttributes objectForKey:NSFileSize] intValue];
	return ret;
}


// --------------------------------------------------------------------------------
+ (NSString*) getDatabaseFormatedFileSize:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	int fileSize = [self getDatabaseFileSize:dataBasePath];
	NSString* extension = [NSString alloc];
	NSString* formatedFileSize = [NSString alloc];
	float ff;
	if (fileSize < 1024)
	{
		formatedFileSize = [NSString stringWithFormat:@"%d", fileSize];
		extension = @"bytes";
	}
	else if (fileSize < 1048576)
	{
		ff = (float) fileSize / 1024.00;
		formatedFileSize = [NSString stringWithFormat:@"%.2f", ff];
		extension = @"Kb";
	}
	else
	{
		ff = (float) fileSize / 1048576;
		formatedFileSize = [NSString stringWithFormat:@"%.2f", ff];
		extension = @"Mb";
	}
	return [NSString stringWithFormat:@"%@ %@", formatedFileSize, extension];
}


// --------------------------------------------------------------------------------
- (BOOL) _connect:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	BOOL ok;
	if (sqlite3_open([dataBasePath UTF8String], &database) != SQLITE_OK) {
		sqlite3_close(database);
		NSAssert1(0, @"Failed to open the database: %@", dataBasePath);
		ok = NO;
	}
	else ok = YES;
	return ok;
}


// --------------------------------------------------------------------------------
- (BOOL) executeQuery:(NSString*)query inDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	BOOL ok = NO;
	if ([self _connect:dataBasePath])
	{
		char *errorMsg;
		if (sqlite3_exec(database, [query UTF8String], NULL, NULL, &errorMsg) != SQLITE_OK)
		{
			sqlite3_close(database);
			NSAssert2(0, @"SQLite Error: %s (%@)", errorMsg, dataBasePath);
		}
		else ok = YES;
		sqlite3_close(database);
	}
	return ok;
}


// --------------------------------------------------------------------------------
- (int) executeScalar:(NSString*)query inDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	int value = 0;
	if ([self _connect:dataBasePath])
	{
		sqlite3_stmt *statement;
		if (sqlite3_prepare_v2(database, [query UTF8String], -1, &statement, nil) == SQLITE_OK)
		{
			while (sqlite3_step(statement) == SQLITE_ROW)
			{
				value = sqlite3_column_int(statement, 0);
				break;
			}
		}
		sqlite3_close(database);
	}
	return value;
}


// --------------------------------------------------------------------------------
- (BOOL) tableExists:(NSString*)tableName inDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSString* query = [NSString stringWithFormat:@"SELECT COUNT(name) FROM sqlite_master WHERE name = '%@';", tableName];
	int is = [self executeScalar:query inDatabase:dataBasePath];	
	return is;
}


// --------------------------------------------------------------------------------
- (BOOL) truncateTable:(NSString*)tableName inDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSString* query = [NSString stringWithFormat:@"DELETE FROM  %@;", tableName];
	return [self executeQuery:query inDatabase:dataBasePath];
}


// --------------------------------------------------------------------------------
- (BOOL) truncateTable:(NSString*)tableName inDatabase:(NSString*)dataBasePath useVacuum:(BOOL)use
// --------------------------------------------------------------------------------
{
	BOOL ok = [self truncateTable:tableName inDatabase:dataBasePath];
	[self executeQuery:@"VACUUM;" inDatabase:dataBasePath];
	return ok;
}


// --------------------------------------------------------------------------------
- (BOOL) dropTable:(NSString*)tableName inDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSString* query = [NSString stringWithFormat:@"DROP TABLE IF EXISTS %@;", tableName];
	return [self executeQuery:query inDatabase:dataBasePath];
}


// --------------------------------------------------------------------------------
- (BOOL) dropTable:(NSString*)tableName inDatabase:(NSString*)dataBasePath useVacuum:(BOOL)use
// --------------------------------------------------------------------------------
{
	BOOL ok = [self dropTable:tableName inDatabase:dataBasePath];
	[self executeQuery:@"VACUUM;" inDatabase:dataBasePath];
	return ok;
}


// --------------------------------------------------------------------------------
- (NSMutableArray *) getTablesInDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSString* query = @"SELECT name FROM sqlite_master WHERE type = 'table' ORDER BY name;";
	//NSMutableArray *data = [self getData:query fromDatabase:dataBasePath];
	NSMutableArray* array = [[NSMutableArray alloc] init];
	if ([self _connect:dataBasePath])
	{
		sqlite3_stmt *statement;
		if (sqlite3_prepare_v2(database, [query UTF8String], -1, &statement, nil) == SQLITE_OK)
		{
			while (sqlite3_step(statement) == SQLITE_ROW)
			{
				char *cName = (char *) sqlite3_column_text(statement, 0);
				NSString* name = [NSString stringWithFormat:@"%s", cName];
				[array addObject:name];
			}
		}
	}
	return array;
}


// --------------------------------------------------------------------------------
- (NSMutableArray*) getData:(NSString*)query fromDatabase:(NSString*)dataBasePath
// --------------------------------------------------------------------------------
{
	NSMutableArray* array = [[NSMutableArray alloc] initWithCapacity:30000];
	if ([self _connect:dataBasePath])
	{
		sqlite3_stmt* statement;
		if (sqlite3_prepare_v2(database, [query UTF8String], -1, &statement, nil) == SQLITE_OK)
		{
			while (sqlite3_step(statement) == SQLITE_ROW)
			{
				NSMutableDictionary* row = [[NSMutableDictionary alloc] init];
				int columns = sqlite3_column_count(statement);
				for (int i = 0; i < columns; i++)
				{
					long type = sqlite3_column_type(statement, i);
					const char* key = sqlite3_column_name(statement, i);
					id value = nil;
					switch(type)
					{
						case SQLITE_INTEGER:
							value = [NSNumber numberWithInt:sqlite3_column_int(statement, i)];
							break;
							
						case SQLITE_FLOAT:
							value = [NSNumber numberWithDouble:sqlite3_column_double(statement, i)];
							break;
							
						case SQLITE_TEXT:
							value = [NSString stringWithCString:(const char*)sqlite3_column_text(statement, i) encoding:NSISOLatin1StringEncoding];
							break;
							
						case SQLITE_BLOB:
							value = [NSString stringWithFormat:@"%d", sqlite3_column_bytes(statement, i)];
							break;
							
						case SQLITE_NULL:
							break;
							
						default:
							break;
					}
					[row setValue:value forKey:[NSString stringWithFormat:@"%s", key]];
				}
				[array addObject:row];
			}
		}
		
		sqlite3_reset(statement);
		sqlite3_finalize(statement);
		sqlite3_close(database);
	}
	return array;
}


@end
